import { createLowlight, all } from 'lowlight';
import * as React from 'react';
import { useRef, useEffect, createContext, useContext, useLayoutEffect, useState, useCallback, memo, Fragment, useMemo, forwardRef, useImperativeHandle } from 'react';
import { createStore, ref } from 'reactivity-store';
import { useSyncExternalStore } from 'use-sync-external-store/shim';
import { flushSync } from 'react-dom';

/**
 * @module LRUCache
 */
const perf = typeof performance === 'object' &&
    performance &&
    typeof performance.now === 'function'
    ? performance
    : Date;
const warned = new Set();
/* c8 ignore start */
const PROCESS = (typeof process === 'object' && !!process ? process : {});
/* c8 ignore start */
const emitWarning = (msg, type, code, fn) => {
    typeof PROCESS.emitWarning === 'function'
        ? PROCESS.emitWarning(msg, type, code, fn)
        : console.error(`[${code}] ${type}: ${msg}`);
};
let AC = globalThis.AbortController;
let AS = globalThis.AbortSignal;
/* c8 ignore start */
if (typeof AC === 'undefined') {
    //@ts-ignore
    AS = class AbortSignal {
        onabort;
        _onabort = [];
        reason;
        aborted = false;
        addEventListener(_, fn) {
            this._onabort.push(fn);
        }
    };
    //@ts-ignore
    AC = class AbortController {
        constructor() {
            warnACPolyfill();
        }
        signal = new AS();
        abort(reason) {
            if (this.signal.aborted)
                return;
            //@ts-ignore
            this.signal.reason = reason;
            //@ts-ignore
            this.signal.aborted = true;
            //@ts-ignore
            for (const fn of this.signal._onabort) {
                fn(reason);
            }
            this.signal.onabort?.(reason);
        }
    };
    let printACPolyfillWarning = PROCESS.env?.LRU_CACHE_IGNORE_AC_WARNING !== '1';
    const warnACPolyfill = () => {
        if (!printACPolyfillWarning)
            return;
        printACPolyfillWarning = false;
        emitWarning('AbortController is not defined. If using lru-cache in ' +
            'node 14, load an AbortController polyfill from the ' +
            '`node-abort-controller` package. A minimal polyfill is ' +
            'provided for use by LRUCache.fetch(), but it should not be ' +
            'relied upon in other contexts (eg, passing it to other APIs that ' +
            'use AbortController/AbortSignal might have undesirable effects). ' +
            'You may disable this with LRU_CACHE_IGNORE_AC_WARNING=1 in the env.', 'NO_ABORT_CONTROLLER', 'ENOTSUP', warnACPolyfill);
    };
}
/* c8 ignore stop */
const shouldWarn = (code) => !warned.has(code);
const isPosInt = (n) => n && n === Math.floor(n) && n > 0 && isFinite(n);
/* c8 ignore start */
// This is a little bit ridiculous, tbh.
// The maximum array length is 2^32-1 or thereabouts on most JS impls.
// And well before that point, you're caching the entire world, I mean,
// that's ~32GB of just integers for the next/prev links, plus whatever
// else to hold that many keys and values.  Just filling the memory with
// zeroes at init time is brutal when you get that big.
// But why not be complete?
// Maybe in the future, these limits will have expanded.
const getUintArray = (max) => !isPosInt(max)
    ? null
    : max <= Math.pow(2, 8)
        ? Uint8Array
        : max <= Math.pow(2, 16)
            ? Uint16Array
            : max <= Math.pow(2, 32)
                ? Uint32Array
                : max <= Number.MAX_SAFE_INTEGER
                    ? ZeroArray
                    : null;
/* c8 ignore stop */
class ZeroArray extends Array {
    constructor(size) {
        super(size);
        this.fill(0);
    }
}
class Stack {
    heap;
    length;
    // private constructor
    static #constructing = false;
    static create(max) {
        const HeapCls = getUintArray(max);
        if (!HeapCls)
            return [];
        Stack.#constructing = true;
        const s = new Stack(max, HeapCls);
        Stack.#constructing = false;
        return s;
    }
    constructor(max, HeapCls) {
        /* c8 ignore start */
        if (!Stack.#constructing) {
            throw new TypeError('instantiate Stack using Stack.create(n)');
        }
        /* c8 ignore stop */
        this.heap = new HeapCls(max);
        this.length = 0;
    }
    push(n) {
        this.heap[this.length++] = n;
    }
    pop() {
        return this.heap[--this.length];
    }
}
/**
 * Default export, the thing you're using this module to get.
 *
 * All properties from the options object (with the exception of
 * {@link OptionsBase.max} and {@link OptionsBase.maxSize}) are added as
 * normal public members. (`max` and `maxBase` are read-only getters.)
 * Changing any of these will alter the defaults for subsequent method calls,
 * but is otherwise safe.
 */
class LRUCache {
    // properties coming in from the options of these, only max and maxSize
    // really *need* to be protected. The rest can be modified, as they just
    // set defaults for various methods.
    #max;
    #maxSize;
    #dispose;
    #disposeAfter;
    #fetchMethod;
    /**
     * {@link LRUCache.OptionsBase.ttl}
     */
    ttl;
    /**
     * {@link LRUCache.OptionsBase.ttlResolution}
     */
    ttlResolution;
    /**
     * {@link LRUCache.OptionsBase.ttlAutopurge}
     */
    ttlAutopurge;
    /**
     * {@link LRUCache.OptionsBase.updateAgeOnGet}
     */
    updateAgeOnGet;
    /**
     * {@link LRUCache.OptionsBase.updateAgeOnHas}
     */
    updateAgeOnHas;
    /**
     * {@link LRUCache.OptionsBase.allowStale}
     */
    allowStale;
    /**
     * {@link LRUCache.OptionsBase.noDisposeOnSet}
     */
    noDisposeOnSet;
    /**
     * {@link LRUCache.OptionsBase.noUpdateTTL}
     */
    noUpdateTTL;
    /**
     * {@link LRUCache.OptionsBase.maxEntrySize}
     */
    maxEntrySize;
    /**
     * {@link LRUCache.OptionsBase.sizeCalculation}
     */
    sizeCalculation;
    /**
     * {@link LRUCache.OptionsBase.noDeleteOnFetchRejection}
     */
    noDeleteOnFetchRejection;
    /**
     * {@link LRUCache.OptionsBase.noDeleteOnStaleGet}
     */
    noDeleteOnStaleGet;
    /**
     * {@link LRUCache.OptionsBase.allowStaleOnFetchAbort}
     */
    allowStaleOnFetchAbort;
    /**
     * {@link LRUCache.OptionsBase.allowStaleOnFetchRejection}
     */
    allowStaleOnFetchRejection;
    /**
     * {@link LRUCache.OptionsBase.ignoreFetchAbort}
     */
    ignoreFetchAbort;
    // computed properties
    #size;
    #calculatedSize;
    #keyMap;
    #keyList;
    #valList;
    #next;
    #prev;
    #head;
    #tail;
    #free;
    #disposed;
    #sizes;
    #starts;
    #ttls;
    #hasDispose;
    #hasFetchMethod;
    #hasDisposeAfter;
    /**
     * Do not call this method unless you need to inspect the
     * inner workings of the cache.  If anything returned by this
     * object is modified in any way, strange breakage may occur.
     *
     * These fields are private for a reason!
     *
     * @internal
     */
    static unsafeExposeInternals(c) {
        return {
            // properties
            starts: c.#starts,
            ttls: c.#ttls,
            sizes: c.#sizes,
            keyMap: c.#keyMap,
            keyList: c.#keyList,
            valList: c.#valList,
            next: c.#next,
            prev: c.#prev,
            get head() {
                return c.#head;
            },
            get tail() {
                return c.#tail;
            },
            free: c.#free,
            // methods
            isBackgroundFetch: (p) => c.#isBackgroundFetch(p),
            backgroundFetch: (k, index, options, context) => c.#backgroundFetch(k, index, options, context),
            moveToTail: (index) => c.#moveToTail(index),
            indexes: (options) => c.#indexes(options),
            rindexes: (options) => c.#rindexes(options),
            isStale: (index) => c.#isStale(index),
        };
    }
    // Protected read-only members
    /**
     * {@link LRUCache.OptionsBase.max} (read-only)
     */
    get max() {
        return this.#max;
    }
    /**
     * {@link LRUCache.OptionsBase.maxSize} (read-only)
     */
    get maxSize() {
        return this.#maxSize;
    }
    /**
     * The total computed size of items in the cache (read-only)
     */
    get calculatedSize() {
        return this.#calculatedSize;
    }
    /**
     * The number of items stored in the cache (read-only)
     */
    get size() {
        return this.#size;
    }
    /**
     * {@link LRUCache.OptionsBase.fetchMethod} (read-only)
     */
    get fetchMethod() {
        return this.#fetchMethod;
    }
    /**
     * {@link LRUCache.OptionsBase.dispose} (read-only)
     */
    get dispose() {
        return this.#dispose;
    }
    /**
     * {@link LRUCache.OptionsBase.disposeAfter} (read-only)
     */
    get disposeAfter() {
        return this.#disposeAfter;
    }
    constructor(options) {
        const { max = 0, ttl, ttlResolution = 1, ttlAutopurge, updateAgeOnGet, updateAgeOnHas, allowStale, dispose, disposeAfter, noDisposeOnSet, noUpdateTTL, maxSize = 0, maxEntrySize = 0, sizeCalculation, fetchMethod, noDeleteOnFetchRejection, noDeleteOnStaleGet, allowStaleOnFetchRejection, allowStaleOnFetchAbort, ignoreFetchAbort, } = options;
        if (max !== 0 && !isPosInt(max)) {
            throw new TypeError('max option must be a nonnegative integer');
        }
        const UintArray = max ? getUintArray(max) : Array;
        if (!UintArray) {
            throw new Error('invalid max value: ' + max);
        }
        this.#max = max;
        this.#maxSize = maxSize;
        this.maxEntrySize = maxEntrySize || this.#maxSize;
        this.sizeCalculation = sizeCalculation;
        if (this.sizeCalculation) {
            if (!this.#maxSize && !this.maxEntrySize) {
                throw new TypeError('cannot set sizeCalculation without setting maxSize or maxEntrySize');
            }
            if (typeof this.sizeCalculation !== 'function') {
                throw new TypeError('sizeCalculation set to non-function');
            }
        }
        if (fetchMethod !== undefined &&
            typeof fetchMethod !== 'function') {
            throw new TypeError('fetchMethod must be a function if specified');
        }
        this.#fetchMethod = fetchMethod;
        this.#hasFetchMethod = !!fetchMethod;
        this.#keyMap = new Map();
        this.#keyList = new Array(max).fill(undefined);
        this.#valList = new Array(max).fill(undefined);
        this.#next = new UintArray(max);
        this.#prev = new UintArray(max);
        this.#head = 0;
        this.#tail = 0;
        this.#free = Stack.create(max);
        this.#size = 0;
        this.#calculatedSize = 0;
        if (typeof dispose === 'function') {
            this.#dispose = dispose;
        }
        if (typeof disposeAfter === 'function') {
            this.#disposeAfter = disposeAfter;
            this.#disposed = [];
        }
        else {
            this.#disposeAfter = undefined;
            this.#disposed = undefined;
        }
        this.#hasDispose = !!this.#dispose;
        this.#hasDisposeAfter = !!this.#disposeAfter;
        this.noDisposeOnSet = !!noDisposeOnSet;
        this.noUpdateTTL = !!noUpdateTTL;
        this.noDeleteOnFetchRejection = !!noDeleteOnFetchRejection;
        this.allowStaleOnFetchRejection = !!allowStaleOnFetchRejection;
        this.allowStaleOnFetchAbort = !!allowStaleOnFetchAbort;
        this.ignoreFetchAbort = !!ignoreFetchAbort;
        // NB: maxEntrySize is set to maxSize if it's set
        if (this.maxEntrySize !== 0) {
            if (this.#maxSize !== 0) {
                if (!isPosInt(this.#maxSize)) {
                    throw new TypeError('maxSize must be a positive integer if specified');
                }
            }
            if (!isPosInt(this.maxEntrySize)) {
                throw new TypeError('maxEntrySize must be a positive integer if specified');
            }
            this.#initializeSizeTracking();
        }
        this.allowStale = !!allowStale;
        this.noDeleteOnStaleGet = !!noDeleteOnStaleGet;
        this.updateAgeOnGet = !!updateAgeOnGet;
        this.updateAgeOnHas = !!updateAgeOnHas;
        this.ttlResolution =
            isPosInt(ttlResolution) || ttlResolution === 0
                ? ttlResolution
                : 1;
        this.ttlAutopurge = !!ttlAutopurge;
        this.ttl = ttl || 0;
        if (this.ttl) {
            if (!isPosInt(this.ttl)) {
                throw new TypeError('ttl must be a positive integer if specified');
            }
            this.#initializeTTLTracking();
        }
        // do not allow completely unbounded caches
        if (this.#max === 0 && this.ttl === 0 && this.#maxSize === 0) {
            throw new TypeError('At least one of max, maxSize, or ttl is required');
        }
        if (!this.ttlAutopurge && !this.#max && !this.#maxSize) {
            const code = 'LRU_CACHE_UNBOUNDED';
            if (shouldWarn(code)) {
                warned.add(code);
                const msg = 'TTL caching without ttlAutopurge, max, or maxSize can ' +
                    'result in unbounded memory consumption.';
                emitWarning(msg, 'UnboundedCacheWarning', code, LRUCache);
            }
        }
    }
    /**
     * Return the remaining TTL time for a given entry key
     */
    getRemainingTTL(key) {
        return this.#keyMap.has(key) ? Infinity : 0;
    }
    #initializeTTLTracking() {
        const ttls = new ZeroArray(this.#max);
        const starts = new ZeroArray(this.#max);
        this.#ttls = ttls;
        this.#starts = starts;
        this.#setItemTTL = (index, ttl, start = perf.now()) => {
            starts[index] = ttl !== 0 ? start : 0;
            ttls[index] = ttl;
            if (ttl !== 0 && this.ttlAutopurge) {
                const t = setTimeout(() => {
                    if (this.#isStale(index)) {
                        this.delete(this.#keyList[index]);
                    }
                }, ttl + 1);
                // unref() not supported on all platforms
                /* c8 ignore start */
                if (t.unref) {
                    t.unref();
                }
                /* c8 ignore stop */
            }
        };
        this.#updateItemAge = index => {
            starts[index] = ttls[index] !== 0 ? perf.now() : 0;
        };
        this.#statusTTL = (status, index) => {
            if (ttls[index]) {
                const ttl = ttls[index];
                const start = starts[index];
                /* c8 ignore next */
                if (!ttl || !start)
                    return;
                status.ttl = ttl;
                status.start = start;
                status.now = cachedNow || getNow();
                const age = status.now - start;
                status.remainingTTL = ttl - age;
            }
        };
        // debounce calls to perf.now() to 1s so we're not hitting
        // that costly call repeatedly.
        let cachedNow = 0;
        const getNow = () => {
            const n = perf.now();
            if (this.ttlResolution > 0) {
                cachedNow = n;
                const t = setTimeout(() => (cachedNow = 0), this.ttlResolution);
                // not available on all platforms
                /* c8 ignore start */
                if (t.unref) {
                    t.unref();
                }
                /* c8 ignore stop */
            }
            return n;
        };
        this.getRemainingTTL = key => {
            const index = this.#keyMap.get(key);
            if (index === undefined) {
                return 0;
            }
            const ttl = ttls[index];
            const start = starts[index];
            if (!ttl || !start) {
                return Infinity;
            }
            const age = (cachedNow || getNow()) - start;
            return ttl - age;
        };
        this.#isStale = index => {
            const s = starts[index];
            const t = ttls[index];
            return !!t && !!s && (cachedNow || getNow()) - s > t;
        };
    }
    // conditionally set private methods related to TTL
    #updateItemAge = () => { };
    #statusTTL = () => { };
    #setItemTTL = () => { };
    /* c8 ignore stop */
    #isStale = () => false;
    #initializeSizeTracking() {
        const sizes = new ZeroArray(this.#max);
        this.#calculatedSize = 0;
        this.#sizes = sizes;
        this.#removeItemSize = index => {
            this.#calculatedSize -= sizes[index];
            sizes[index] = 0;
        };
        this.#requireSize = (k, v, size, sizeCalculation) => {
            // provisionally accept background fetches.
            // actual value size will be checked when they return.
            if (this.#isBackgroundFetch(v)) {
                return 0;
            }
            if (!isPosInt(size)) {
                if (sizeCalculation) {
                    if (typeof sizeCalculation !== 'function') {
                        throw new TypeError('sizeCalculation must be a function');
                    }
                    size = sizeCalculation(v, k);
                    if (!isPosInt(size)) {
                        throw new TypeError('sizeCalculation return invalid (expect positive integer)');
                    }
                }
                else {
                    throw new TypeError('invalid size value (must be positive integer). ' +
                        'When maxSize or maxEntrySize is used, sizeCalculation ' +
                        'or size must be set.');
                }
            }
            return size;
        };
        this.#addItemSize = (index, size, status) => {
            sizes[index] = size;
            if (this.#maxSize) {
                const maxSize = this.#maxSize - sizes[index];
                while (this.#calculatedSize > maxSize) {
                    this.#evict(true);
                }
            }
            this.#calculatedSize += sizes[index];
            if (status) {
                status.entrySize = size;
                status.totalCalculatedSize = this.#calculatedSize;
            }
        };
    }
    #removeItemSize = _i => { };
    #addItemSize = (_i, _s, _st) => { };
    #requireSize = (_k, _v, size, sizeCalculation) => {
        if (size || sizeCalculation) {
            throw new TypeError('cannot set size without setting maxSize or maxEntrySize on cache');
        }
        return 0;
    };
    *#indexes({ allowStale = this.allowStale } = {}) {
        if (this.#size) {
            for (let i = this.#tail; true;) {
                if (!this.#isValidIndex(i)) {
                    break;
                }
                if (allowStale || !this.#isStale(i)) {
                    yield i;
                }
                if (i === this.#head) {
                    break;
                }
                else {
                    i = this.#prev[i];
                }
            }
        }
    }
    *#rindexes({ allowStale = this.allowStale } = {}) {
        if (this.#size) {
            for (let i = this.#head; true;) {
                if (!this.#isValidIndex(i)) {
                    break;
                }
                if (allowStale || !this.#isStale(i)) {
                    yield i;
                }
                if (i === this.#tail) {
                    break;
                }
                else {
                    i = this.#next[i];
                }
            }
        }
    }
    #isValidIndex(index) {
        return (index !== undefined &&
            this.#keyMap.get(this.#keyList[index]) === index);
    }
    /**
     * Return a generator yielding `[key, value]` pairs,
     * in order from most recently used to least recently used.
     */
    *entries() {
        for (const i of this.#indexes()) {
            if (this.#valList[i] !== undefined &&
                this.#keyList[i] !== undefined &&
                !this.#isBackgroundFetch(this.#valList[i])) {
                yield [this.#keyList[i], this.#valList[i]];
            }
        }
    }
    /**
     * Inverse order version of {@link LRUCache.entries}
     *
     * Return a generator yielding `[key, value]` pairs,
     * in order from least recently used to most recently used.
     */
    *rentries() {
        for (const i of this.#rindexes()) {
            if (this.#valList[i] !== undefined &&
                this.#keyList[i] !== undefined &&
                !this.#isBackgroundFetch(this.#valList[i])) {
                yield [this.#keyList[i], this.#valList[i]];
            }
        }
    }
    /**
     * Return a generator yielding the keys in the cache,
     * in order from most recently used to least recently used.
     */
    *keys() {
        for (const i of this.#indexes()) {
            const k = this.#keyList[i];
            if (k !== undefined &&
                !this.#isBackgroundFetch(this.#valList[i])) {
                yield k;
            }
        }
    }
    /**
     * Inverse order version of {@link LRUCache.keys}
     *
     * Return a generator yielding the keys in the cache,
     * in order from least recently used to most recently used.
     */
    *rkeys() {
        for (const i of this.#rindexes()) {
            const k = this.#keyList[i];
            if (k !== undefined &&
                !this.#isBackgroundFetch(this.#valList[i])) {
                yield k;
            }
        }
    }
    /**
     * Return a generator yielding the values in the cache,
     * in order from most recently used to least recently used.
     */
    *values() {
        for (const i of this.#indexes()) {
            const v = this.#valList[i];
            if (v !== undefined &&
                !this.#isBackgroundFetch(this.#valList[i])) {
                yield this.#valList[i];
            }
        }
    }
    /**
     * Inverse order version of {@link LRUCache.values}
     *
     * Return a generator yielding the values in the cache,
     * in order from least recently used to most recently used.
     */
    *rvalues() {
        for (const i of this.#rindexes()) {
            const v = this.#valList[i];
            if (v !== undefined &&
                !this.#isBackgroundFetch(this.#valList[i])) {
                yield this.#valList[i];
            }
        }
    }
    /**
     * Iterating over the cache itself yields the same results as
     * {@link LRUCache.entries}
     */
    [Symbol.iterator]() {
        return this.entries();
    }
    /**
     * A String value that is used in the creation of the default string description of an object.
     * Called by the built-in method Object.prototype.toString.
     */
    [Symbol.toStringTag] = 'LRUCache';
    /**
     * Find a value for which the supplied fn method returns a truthy value,
     * similar to Array.find().  fn is called as fn(value, key, cache).
     */
    find(fn, getOptions = {}) {
        for (const i of this.#indexes()) {
            const v = this.#valList[i];
            const value = this.#isBackgroundFetch(v)
                ? v.__staleWhileFetching
                : v;
            if (value === undefined)
                continue;
            if (fn(value, this.#keyList[i], this)) {
                return this.get(this.#keyList[i], getOptions);
            }
        }
    }
    /**
     * Call the supplied function on each item in the cache, in order from
     * most recently used to least recently used.  fn is called as
     * fn(value, key, cache).  Does not update age or recenty of use.
     * Does not iterate over stale values.
     */
    forEach(fn, thisp = this) {
        for (const i of this.#indexes()) {
            const v = this.#valList[i];
            const value = this.#isBackgroundFetch(v)
                ? v.__staleWhileFetching
                : v;
            if (value === undefined)
                continue;
            fn.call(thisp, value, this.#keyList[i], this);
        }
    }
    /**
     * The same as {@link LRUCache.forEach} but items are iterated over in
     * reverse order.  (ie, less recently used items are iterated over first.)
     */
    rforEach(fn, thisp = this) {
        for (const i of this.#rindexes()) {
            const v = this.#valList[i];
            const value = this.#isBackgroundFetch(v)
                ? v.__staleWhileFetching
                : v;
            if (value === undefined)
                continue;
            fn.call(thisp, value, this.#keyList[i], this);
        }
    }
    /**
     * Delete any stale entries. Returns true if anything was removed,
     * false otherwise.
     */
    purgeStale() {
        let deleted = false;
        for (const i of this.#rindexes({ allowStale: true })) {
            if (this.#isStale(i)) {
                this.delete(this.#keyList[i]);
                deleted = true;
            }
        }
        return deleted;
    }
    /**
     * Get the extended info about a given entry, to get its value, size, and
     * TTL info simultaneously. Like {@link LRUCache#dump}, but just for a
     * single key. Always returns stale values, if their info is found in the
     * cache, so be sure to check for expired TTLs if relevant.
     */
    info(key) {
        const i = this.#keyMap.get(key);
        if (i === undefined)
            return undefined;
        const v = this.#valList[i];
        const value = this.#isBackgroundFetch(v)
            ? v.__staleWhileFetching
            : v;
        if (value === undefined)
            return undefined;
        const entry = { value };
        if (this.#ttls && this.#starts) {
            const ttl = this.#ttls[i];
            const start = this.#starts[i];
            if (ttl && start) {
                const remain = ttl - (perf.now() - start);
                entry.ttl = remain;
                entry.start = Date.now();
            }
        }
        if (this.#sizes) {
            entry.size = this.#sizes[i];
        }
        return entry;
    }
    /**
     * Return an array of [key, {@link LRUCache.Entry}] tuples which can be
     * passed to cache.load()
     */
    dump() {
        const arr = [];
        for (const i of this.#indexes({ allowStale: true })) {
            const key = this.#keyList[i];
            const v = this.#valList[i];
            const value = this.#isBackgroundFetch(v)
                ? v.__staleWhileFetching
                : v;
            if (value === undefined || key === undefined)
                continue;
            const entry = { value };
            if (this.#ttls && this.#starts) {
                entry.ttl = this.#ttls[i];
                // always dump the start relative to a portable timestamp
                // it's ok for this to be a bit slow, it's a rare operation.
                const age = perf.now() - this.#starts[i];
                entry.start = Math.floor(Date.now() - age);
            }
            if (this.#sizes) {
                entry.size = this.#sizes[i];
            }
            arr.unshift([key, entry]);
        }
        return arr;
    }
    /**
     * Reset the cache and load in the items in entries in the order listed.
     * Note that the shape of the resulting cache may be different if the
     * same options are not used in both caches.
     */
    load(arr) {
        this.clear();
        for (const [key, entry] of arr) {
            if (entry.start) {
                // entry.start is a portable timestamp, but we may be using
                // node's performance.now(), so calculate the offset, so that
                // we get the intended remaining TTL, no matter how long it's
                // been on ice.
                //
                // it's ok for this to be a bit slow, it's a rare operation.
                const age = Date.now() - entry.start;
                entry.start = perf.now() - age;
            }
            this.set(key, entry.value, entry);
        }
    }
    /**
     * Add a value to the cache.
     *
     * Note: if `undefined` is specified as a value, this is an alias for
     * {@link LRUCache#delete}
     */
    set(k, v, setOptions = {}) {
        if (v === undefined) {
            this.delete(k);
            return this;
        }
        const { ttl = this.ttl, start, noDisposeOnSet = this.noDisposeOnSet, sizeCalculation = this.sizeCalculation, status, } = setOptions;
        let { noUpdateTTL = this.noUpdateTTL } = setOptions;
        const size = this.#requireSize(k, v, setOptions.size || 0, sizeCalculation);
        // if the item doesn't fit, don't do anything
        // NB: maxEntrySize set to maxSize by default
        if (this.maxEntrySize && size > this.maxEntrySize) {
            if (status) {
                status.set = 'miss';
                status.maxEntrySizeExceeded = true;
            }
            // have to delete, in case something is there already.
            this.delete(k);
            return this;
        }
        let index = this.#size === 0 ? undefined : this.#keyMap.get(k);
        if (index === undefined) {
            // addition
            index = (this.#size === 0
                ? this.#tail
                : this.#free.length !== 0
                    ? this.#free.pop()
                    : this.#size === this.#max
                        ? this.#evict(false)
                        : this.#size);
            this.#keyList[index] = k;
            this.#valList[index] = v;
            this.#keyMap.set(k, index);
            this.#next[this.#tail] = index;
            this.#prev[index] = this.#tail;
            this.#tail = index;
            this.#size++;
            this.#addItemSize(index, size, status);
            if (status)
                status.set = 'add';
            noUpdateTTL = false;
        }
        else {
            // update
            this.#moveToTail(index);
            const oldVal = this.#valList[index];
            if (v !== oldVal) {
                if (this.#hasFetchMethod && this.#isBackgroundFetch(oldVal)) {
                    oldVal.__abortController.abort(new Error('replaced'));
                    const { __staleWhileFetching: s } = oldVal;
                    if (s !== undefined && !noDisposeOnSet) {
                        if (this.#hasDispose) {
                            this.#dispose?.(s, k, 'set');
                        }
                        if (this.#hasDisposeAfter) {
                            this.#disposed?.push([s, k, 'set']);
                        }
                    }
                }
                else if (!noDisposeOnSet) {
                    if (this.#hasDispose) {
                        this.#dispose?.(oldVal, k, 'set');
                    }
                    if (this.#hasDisposeAfter) {
                        this.#disposed?.push([oldVal, k, 'set']);
                    }
                }
                this.#removeItemSize(index);
                this.#addItemSize(index, size, status);
                this.#valList[index] = v;
                if (status) {
                    status.set = 'replace';
                    const oldValue = oldVal && this.#isBackgroundFetch(oldVal)
                        ? oldVal.__staleWhileFetching
                        : oldVal;
                    if (oldValue !== undefined)
                        status.oldValue = oldValue;
                }
            }
            else if (status) {
                status.set = 'update';
            }
        }
        if (ttl !== 0 && !this.#ttls) {
            this.#initializeTTLTracking();
        }
        if (this.#ttls) {
            if (!noUpdateTTL) {
                this.#setItemTTL(index, ttl, start);
            }
            if (status)
                this.#statusTTL(status, index);
        }
        if (!noDisposeOnSet && this.#hasDisposeAfter && this.#disposed) {
            const dt = this.#disposed;
            let task;
            while ((task = dt?.shift())) {
                this.#disposeAfter?.(...task);
            }
        }
        return this;
    }
    /**
     * Evict the least recently used item, returning its value or
     * `undefined` if cache is empty.
     */
    pop() {
        try {
            while (this.#size) {
                const val = this.#valList[this.#head];
                this.#evict(true);
                if (this.#isBackgroundFetch(val)) {
                    if (val.__staleWhileFetching) {
                        return val.__staleWhileFetching;
                    }
                }
                else if (val !== undefined) {
                    return val;
                }
            }
        }
        finally {
            if (this.#hasDisposeAfter && this.#disposed) {
                const dt = this.#disposed;
                let task;
                while ((task = dt?.shift())) {
                    this.#disposeAfter?.(...task);
                }
            }
        }
    }
    #evict(free) {
        const head = this.#head;
        const k = this.#keyList[head];
        const v = this.#valList[head];
        if (this.#hasFetchMethod && this.#isBackgroundFetch(v)) {
            v.__abortController.abort(new Error('evicted'));
        }
        else if (this.#hasDispose || this.#hasDisposeAfter) {
            if (this.#hasDispose) {
                this.#dispose?.(v, k, 'evict');
            }
            if (this.#hasDisposeAfter) {
                this.#disposed?.push([v, k, 'evict']);
            }
        }
        this.#removeItemSize(head);
        // if we aren't about to use the index, then null these out
        if (free) {
            this.#keyList[head] = undefined;
            this.#valList[head] = undefined;
            this.#free.push(head);
        }
        if (this.#size === 1) {
            this.#head = this.#tail = 0;
            this.#free.length = 0;
        }
        else {
            this.#head = this.#next[head];
        }
        this.#keyMap.delete(k);
        this.#size--;
        return head;
    }
    /**
     * Check if a key is in the cache, without updating the recency of use.
     * Will return false if the item is stale, even though it is technically
     * in the cache.
     *
     * Will not update item age unless
     * {@link LRUCache.OptionsBase.updateAgeOnHas} is set.
     */
    has(k, hasOptions = {}) {
        const { updateAgeOnHas = this.updateAgeOnHas, status } = hasOptions;
        const index = this.#keyMap.get(k);
        if (index !== undefined) {
            const v = this.#valList[index];
            if (this.#isBackgroundFetch(v) &&
                v.__staleWhileFetching === undefined) {
                return false;
            }
            if (!this.#isStale(index)) {
                if (updateAgeOnHas) {
                    this.#updateItemAge(index);
                }
                if (status) {
                    status.has = 'hit';
                    this.#statusTTL(status, index);
                }
                return true;
            }
            else if (status) {
                status.has = 'stale';
                this.#statusTTL(status, index);
            }
        }
        else if (status) {
            status.has = 'miss';
        }
        return false;
    }
    /**
     * Like {@link LRUCache#get} but doesn't update recency or delete stale
     * items.
     *
     * Returns `undefined` if the item is stale, unless
     * {@link LRUCache.OptionsBase.allowStale} is set.
     */
    peek(k, peekOptions = {}) {
        const { allowStale = this.allowStale } = peekOptions;
        const index = this.#keyMap.get(k);
        if (index === undefined ||
            (!allowStale && this.#isStale(index))) {
            return;
        }
        const v = this.#valList[index];
        // either stale and allowed, or forcing a refresh of non-stale value
        return this.#isBackgroundFetch(v) ? v.__staleWhileFetching : v;
    }
    #backgroundFetch(k, index, options, context) {
        const v = index === undefined ? undefined : this.#valList[index];
        if (this.#isBackgroundFetch(v)) {
            return v;
        }
        const ac = new AC();
        const { signal } = options;
        // when/if our AC signals, then stop listening to theirs.
        signal?.addEventListener('abort', () => ac.abort(signal.reason), {
            signal: ac.signal,
        });
        const fetchOpts = {
            signal: ac.signal,
            options,
            context,
        };
        const cb = (v, updateCache = false) => {
            const { aborted } = ac.signal;
            const ignoreAbort = options.ignoreFetchAbort && v !== undefined;
            if (options.status) {
                if (aborted && !updateCache) {
                    options.status.fetchAborted = true;
                    options.status.fetchError = ac.signal.reason;
                    if (ignoreAbort)
                        options.status.fetchAbortIgnored = true;
                }
                else {
                    options.status.fetchResolved = true;
                }
            }
            if (aborted && !ignoreAbort && !updateCache) {
                return fetchFail(ac.signal.reason);
            }
            // either we didn't abort, and are still here, or we did, and ignored
            const bf = p;
            if (this.#valList[index] === p) {
                if (v === undefined) {
                    if (bf.__staleWhileFetching) {
                        this.#valList[index] = bf.__staleWhileFetching;
                    }
                    else {
                        this.delete(k);
                    }
                }
                else {
                    if (options.status)
                        options.status.fetchUpdated = true;
                    this.set(k, v, fetchOpts.options);
                }
            }
            return v;
        };
        const eb = (er) => {
            if (options.status) {
                options.status.fetchRejected = true;
                options.status.fetchError = er;
            }
            return fetchFail(er);
        };
        const fetchFail = (er) => {
            const { aborted } = ac.signal;
            const allowStaleAborted = aborted && options.allowStaleOnFetchAbort;
            const allowStale = allowStaleAborted || options.allowStaleOnFetchRejection;
            const noDelete = allowStale || options.noDeleteOnFetchRejection;
            const bf = p;
            if (this.#valList[index] === p) {
                // if we allow stale on fetch rejections, then we need to ensure that
                // the stale value is not removed from the cache when the fetch fails.
                const del = !noDelete || bf.__staleWhileFetching === undefined;
                if (del) {
                    this.delete(k);
                }
                else if (!allowStaleAborted) {
                    // still replace the *promise* with the stale value,
                    // since we are done with the promise at this point.
                    // leave it untouched if we're still waiting for an
                    // aborted background fetch that hasn't yet returned.
                    this.#valList[index] = bf.__staleWhileFetching;
                }
            }
            if (allowStale) {
                if (options.status && bf.__staleWhileFetching !== undefined) {
                    options.status.returnedStale = true;
                }
                return bf.__staleWhileFetching;
            }
            else if (bf.__returned === bf) {
                throw er;
            }
        };
        const pcall = (res, rej) => {
            const fmp = this.#fetchMethod?.(k, v, fetchOpts);
            if (fmp && fmp instanceof Promise) {
                fmp.then(v => res(v === undefined ? undefined : v), rej);
            }
            // ignored, we go until we finish, regardless.
            // defer check until we are actually aborting,
            // so fetchMethod can override.
            ac.signal.addEventListener('abort', () => {
                if (!options.ignoreFetchAbort ||
                    options.allowStaleOnFetchAbort) {
                    res(undefined);
                    // when it eventually resolves, update the cache.
                    if (options.allowStaleOnFetchAbort) {
                        res = v => cb(v, true);
                    }
                }
            });
        };
        if (options.status)
            options.status.fetchDispatched = true;
        const p = new Promise(pcall).then(cb, eb);
        const bf = Object.assign(p, {
            __abortController: ac,
            __staleWhileFetching: v,
            __returned: undefined,
        });
        if (index === undefined) {
            // internal, don't expose status.
            this.set(k, bf, { ...fetchOpts.options, status: undefined });
            index = this.#keyMap.get(k);
        }
        else {
            this.#valList[index] = bf;
        }
        return bf;
    }
    #isBackgroundFetch(p) {
        if (!this.#hasFetchMethod)
            return false;
        const b = p;
        return (!!b &&
            b instanceof Promise &&
            b.hasOwnProperty('__staleWhileFetching') &&
            b.__abortController instanceof AC);
    }
    async fetch(k, fetchOptions = {}) {
        const { 
        // get options
        allowStale = this.allowStale, updateAgeOnGet = this.updateAgeOnGet, noDeleteOnStaleGet = this.noDeleteOnStaleGet, 
        // set options
        ttl = this.ttl, noDisposeOnSet = this.noDisposeOnSet, size = 0, sizeCalculation = this.sizeCalculation, noUpdateTTL = this.noUpdateTTL, 
        // fetch exclusive options
        noDeleteOnFetchRejection = this.noDeleteOnFetchRejection, allowStaleOnFetchRejection = this.allowStaleOnFetchRejection, ignoreFetchAbort = this.ignoreFetchAbort, allowStaleOnFetchAbort = this.allowStaleOnFetchAbort, context, forceRefresh = false, status, signal, } = fetchOptions;
        if (!this.#hasFetchMethod) {
            if (status)
                status.fetch = 'get';
            return this.get(k, {
                allowStale,
                updateAgeOnGet,
                noDeleteOnStaleGet,
                status,
            });
        }
        const options = {
            allowStale,
            updateAgeOnGet,
            noDeleteOnStaleGet,
            ttl,
            noDisposeOnSet,
            size,
            sizeCalculation,
            noUpdateTTL,
            noDeleteOnFetchRejection,
            allowStaleOnFetchRejection,
            allowStaleOnFetchAbort,
            ignoreFetchAbort,
            status,
            signal,
        };
        let index = this.#keyMap.get(k);
        if (index === undefined) {
            if (status)
                status.fetch = 'miss';
            const p = this.#backgroundFetch(k, index, options, context);
            return (p.__returned = p);
        }
        else {
            // in cache, maybe already fetching
            const v = this.#valList[index];
            if (this.#isBackgroundFetch(v)) {
                const stale = allowStale && v.__staleWhileFetching !== undefined;
                if (status) {
                    status.fetch = 'inflight';
                    if (stale)
                        status.returnedStale = true;
                }
                return stale ? v.__staleWhileFetching : (v.__returned = v);
            }
            // if we force a refresh, that means do NOT serve the cached value,
            // unless we are already in the process of refreshing the cache.
            const isStale = this.#isStale(index);
            if (!forceRefresh && !isStale) {
                if (status)
                    status.fetch = 'hit';
                this.#moveToTail(index);
                if (updateAgeOnGet) {
                    this.#updateItemAge(index);
                }
                if (status)
                    this.#statusTTL(status, index);
                return v;
            }
            // ok, it is stale or a forced refresh, and not already fetching.
            // refresh the cache.
            const p = this.#backgroundFetch(k, index, options, context);
            const hasStale = p.__staleWhileFetching !== undefined;
            const staleVal = hasStale && allowStale;
            if (status) {
                status.fetch = isStale ? 'stale' : 'refresh';
                if (staleVal && isStale)
                    status.returnedStale = true;
            }
            return staleVal ? p.__staleWhileFetching : (p.__returned = p);
        }
    }
    /**
     * Return a value from the cache. Will update the recency of the cache
     * entry found.
     *
     * If the key is not found, get() will return `undefined`.
     */
    get(k, getOptions = {}) {
        const { allowStale = this.allowStale, updateAgeOnGet = this.updateAgeOnGet, noDeleteOnStaleGet = this.noDeleteOnStaleGet, status, } = getOptions;
        const index = this.#keyMap.get(k);
        if (index !== undefined) {
            const value = this.#valList[index];
            const fetching = this.#isBackgroundFetch(value);
            if (status)
                this.#statusTTL(status, index);
            if (this.#isStale(index)) {
                if (status)
                    status.get = 'stale';
                // delete only if not an in-flight background fetch
                if (!fetching) {
                    if (!noDeleteOnStaleGet) {
                        this.delete(k);
                    }
                    if (status && allowStale)
                        status.returnedStale = true;
                    return allowStale ? value : undefined;
                }
                else {
                    if (status &&
                        allowStale &&
                        value.__staleWhileFetching !== undefined) {
                        status.returnedStale = true;
                    }
                    return allowStale ? value.__staleWhileFetching : undefined;
                }
            }
            else {
                if (status)
                    status.get = 'hit';
                // if we're currently fetching it, we don't actually have it yet
                // it's not stale, which means this isn't a staleWhileRefetching.
                // If it's not stale, and fetching, AND has a __staleWhileFetching
                // value, then that means the user fetched with {forceRefresh:true},
                // so it's safe to return that value.
                if (fetching) {
                    return value.__staleWhileFetching;
                }
                this.#moveToTail(index);
                if (updateAgeOnGet) {
                    this.#updateItemAge(index);
                }
                return value;
            }
        }
        else if (status) {
            status.get = 'miss';
        }
    }
    #connect(p, n) {
        this.#prev[n] = p;
        this.#next[p] = n;
    }
    #moveToTail(index) {
        // if tail already, nothing to do
        // if head, move head to next[index]
        // else
        //   move next[prev[index]] to next[index] (head has no prev)
        //   move prev[next[index]] to prev[index]
        // prev[index] = tail
        // next[tail] = index
        // tail = index
        if (index !== this.#tail) {
            if (index === this.#head) {
                this.#head = this.#next[index];
            }
            else {
                this.#connect(this.#prev[index], this.#next[index]);
            }
            this.#connect(this.#tail, index);
            this.#tail = index;
        }
    }
    /**
     * Deletes a key out of the cache.
     * Returns true if the key was deleted, false otherwise.
     */
    delete(k) {
        let deleted = false;
        if (this.#size !== 0) {
            const index = this.#keyMap.get(k);
            if (index !== undefined) {
                deleted = true;
                if (this.#size === 1) {
                    this.clear();
                }
                else {
                    this.#removeItemSize(index);
                    const v = this.#valList[index];
                    if (this.#isBackgroundFetch(v)) {
                        v.__abortController.abort(new Error('deleted'));
                    }
                    else if (this.#hasDispose || this.#hasDisposeAfter) {
                        if (this.#hasDispose) {
                            this.#dispose?.(v, k, 'delete');
                        }
                        if (this.#hasDisposeAfter) {
                            this.#disposed?.push([v, k, 'delete']);
                        }
                    }
                    this.#keyMap.delete(k);
                    this.#keyList[index] = undefined;
                    this.#valList[index] = undefined;
                    if (index === this.#tail) {
                        this.#tail = this.#prev[index];
                    }
                    else if (index === this.#head) {
                        this.#head = this.#next[index];
                    }
                    else {
                        const pi = this.#prev[index];
                        this.#next[pi] = this.#next[index];
                        const ni = this.#next[index];
                        this.#prev[ni] = this.#prev[index];
                    }
                    this.#size--;
                    this.#free.push(index);
                }
            }
        }
        if (this.#hasDisposeAfter && this.#disposed?.length) {
            const dt = this.#disposed;
            let task;
            while ((task = dt?.shift())) {
                this.#disposeAfter?.(...task);
            }
        }
        return deleted;
    }
    /**
     * Clear the cache entirely, throwing away all values.
     */
    clear() {
        for (const index of this.#rindexes({ allowStale: true })) {
            const v = this.#valList[index];
            if (this.#isBackgroundFetch(v)) {
                v.__abortController.abort(new Error('deleted'));
            }
            else {
                const k = this.#keyList[index];
                if (this.#hasDispose) {
                    this.#dispose?.(v, k, 'delete');
                }
                if (this.#hasDisposeAfter) {
                    this.#disposed?.push([v, k, 'delete']);
                }
            }
        }
        this.#keyMap.clear();
        this.#valList.fill(undefined);
        this.#keyList.fill(undefined);
        if (this.#ttls && this.#starts) {
            this.#ttls.fill(0);
            this.#starts.fill(0);
        }
        if (this.#sizes) {
            this.#sizes.fill(0);
        }
        this.#head = 0;
        this.#tail = 0;
        this.#free.length = 0;
        this.#calculatedSize = 0;
        this.#size = 0;
        if (this.#hasDisposeAfter && this.#disposed) {
            const dt = this.#disposed;
            let task;
            while ((task = dt?.shift())) {
                this.#disposeAfter?.(...task);
            }
        }
    }
}

const lowlight = createLowlight(all);
lowlight.register("vue", function hljsDefineVue(hljs) {
    return {
        subLanguage: "xml",
        contains: [
            hljs.COMMENT("<!--", "-->", {
                relevance: 10,
            }),
            {
                begin: /^(\s*)(<script>)/gm,
                end: /^(\s*)(<\/script>)/gm,
                subLanguage: "javascript",
                excludeBegin: true,
                excludeEnd: true,
            },
            {
                begin: /^(?:\s*)(?:<script\s+lang=(["'])ts\1>)/gm,
                end: /^(\s*)(<\/script>)/gm,
                subLanguage: "typescript",
                excludeBegin: true,
                excludeEnd: true,
            },
            {
                begin: /^(\s*)(<style(\s+scoped)?>)/gm,
                end: /^(\s*)(<\/style>)/gm,
                subLanguage: "css",
                excludeBegin: true,
                excludeEnd: true,
            },
            {
                begin: /^(?:\s*)(?:<style(?:\s+scoped)?\s+lang=(["'])(?:s[ca]ss)\1(?:\s+scoped)?>)/gm,
                end: /^(\s*)(<\/style>)/gm,
                subLanguage: "scss",
                excludeBegin: true,
                excludeEnd: true,
            },
            {
                begin: /^(?:\s*)(?:<style(?:\s+scoped)?\s+lang=(["'])stylus\1(?:\s+scoped)?>)/gm,
                end: /^(\s*)(<\/style>)/gm,
                subLanguage: "stylus",
                excludeBegin: true,
                excludeEnd: true,
            },
        ],
    };
});
const highlighter = lowlight;
let _autoDetectLang = true;
let _maxLineToIgnoreSyntax = 2000;
const _ignoreSyntaxHighlightList = [];
Object.defineProperty(highlighter, "maxLineToIgnoreSyntax", {
    get: () => _maxLineToIgnoreSyntax,
});
Object.defineProperty(highlighter, "setMaxLineToIgnoreSyntax", {
    value: (v) => {
        _maxLineToIgnoreSyntax = v;
    },
});
Object.defineProperty(highlighter, "autoDetectLang", {
    get: () => _autoDetectLang,
});
Object.defineProperty(highlighter, "setAutoDetectLang", {
    value: (v) => {
        _autoDetectLang = v;
    },
});
Object.defineProperty(highlighter, "ignoreSyntaxHighlightList", {
    get: () => _ignoreSyntaxHighlightList,
});
Object.defineProperty(highlighter, "setIgnoreSyntaxHighlightList", {
    value: (v) => {
        _ignoreSyntaxHighlightList.length = 0;
        _ignoreSyntaxHighlightList.push(...v);
    },
});

const map = new LRUCache({ max: 30 });
class File {
    raw;
    lang;
    fileName;
    ast;
    rawFile = {};
    hasDoRaw = false;
    rawLength;
    syntaxFile = {};
    hasDoSyntax = false;
    syntaxLength;
    maxLineNumber = 0;
    constructor(raw, lang, fileName) {
        this.raw = raw;
        this.lang = lang;
        this.fileName = fileName;
        Object.defineProperty(this, "__v_skip", { value: true });
    }
    doSyntax({ autoDetectLang, registerHighlighter, }) {
        if (!this.raw || this.hasDoSyntax)
            return;
        let hasRegisteredLang = true;
        const _highlighter = registerHighlighter || highlighter;
        if (this.syntaxLength) {
            process.env.NODE_ENV === "development" && console.error("current file already doSyntax before!");
            return;
        }
        if (!_highlighter.registered(this.lang)) {
            hasRegisteredLang = false;
            if (!autoDetectLang) {
                process.env.NODE_ENV === "development" && console.warn(`not support current lang: ${this.lang} yet`);
                return;
            }
        }
        if (this.rawLength > _highlighter.maxLineToIgnoreSyntax) {
            process.env.NODE_ENV === "development" && console.warn(`ignore syntax for current file, because the rawLength is too long: ${this.rawLength}`);
            return;
        }
        if (this.fileName &&
            _highlighter.ignoreSyntaxHighlightList.some((item) => item instanceof RegExp ? item.test(this.fileName) : this.fileName === item)) {
            process.env.NODE_ENV === "development" &&
                console.warn(`ignore syntax for current file, because the fileName is in the ignoreSyntaxHighlightList: ${this.fileName}`);
            return;
        }
        if (hasRegisteredLang) {
            this.ast = _highlighter.highlight(this.lang, this.raw);
        }
        else {
            this.ast = _highlighter.highlightAuto(this.raw);
        }
        this.#doAST();
        if (process.env.NODE_ENV === "development") {
            this.#doCheck();
        }
        this.hasDoSyntax = true;
    }
    doRaw() {
        if (!this.raw || this.hasDoRaw)
            return;
        const rawString = this.raw;
        const rawArray = rawString.split("\n");
        this.rawLength = rawArray.length;
        this.maxLineNumber = rawArray.length;
        this.rawFile = {};
        for (let i = 0; i < rawArray.length; i++) {
            this.rawFile[i + 1] = i < rawArray.length - 1 ? rawArray[i] + "\n" : rawArray[i];
        }
        // reduce 对于大数组性能很差
        // this.rawFile = rawArray.reduce(
        //   (p, item, index) => ({
        //     ...p,
        //     [index + 1]: index < rawArray.length - 1 ? item + "\n" : item,
        //   }),
        //   {}
        // );
        this.hasDoRaw = true;
    }
    #doAST() {
        const ast = this.ast;
        let lineNumber = 1;
        const syntaxObj = this.syntaxFile;
        const loopAST = (nodes, wrapper) => {
            nodes.forEach((node) => {
                if (node.type === "text") {
                    if (node.value.indexOf("\n") === -1) {
                        const valueLength = node.value.length;
                        if (!syntaxObj[lineNumber]) {
                            node.startIndex = 0;
                            node.endIndex = valueLength - 1;
                            const item = {
                                value: node.value,
                                lineNumber,
                                valueLength,
                                nodeList: [{ node, wrapper }],
                            };
                            syntaxObj[lineNumber] = item;
                        }
                        else {
                            node.startIndex = syntaxObj[lineNumber].valueLength;
                            node.endIndex = node.startIndex + valueLength - 1;
                            syntaxObj[lineNumber].value += node.value;
                            syntaxObj[lineNumber].valueLength += valueLength;
                            syntaxObj[lineNumber].nodeList.push({ node, wrapper });
                        }
                        node.lineNumber = lineNumber;
                        return;
                    }
                    const lines = node.value.split("\n");
                    node.children = node.children || [];
                    for (let i = 0; i < lines.length; i++) {
                        const _value = i === lines.length - 1 ? lines[i] : lines[i] + "\n";
                        const _lineNumber = i === 0 ? lineNumber : ++lineNumber;
                        const _valueLength = _value.length;
                        const _node = {
                            type: "text",
                            value: _value,
                            startIndex: Infinity,
                            endIndex: Infinity,
                            lineNumber: _lineNumber,
                        };
                        if (!syntaxObj[_lineNumber]) {
                            _node.startIndex = 0;
                            _node.endIndex = _valueLength - 1;
                            const item = {
                                value: _value,
                                lineNumber: _lineNumber,
                                valueLength: _valueLength,
                                nodeList: [{ node: _node, wrapper }],
                            };
                            syntaxObj[_lineNumber] = item;
                        }
                        else {
                            _node.startIndex = syntaxObj[_lineNumber].valueLength;
                            _node.endIndex = _node.startIndex + _valueLength - 1;
                            syntaxObj[_lineNumber].value += _value;
                            syntaxObj[_lineNumber].valueLength += _valueLength;
                            syntaxObj[_lineNumber].nodeList.push({ node: _node, wrapper });
                        }
                        node.children.push(_node);
                    }
                    node.lineNumber = lineNumber;
                    return;
                }
                if (node.children) {
                    loopAST(node.children, node);
                    node.lineNumber = lineNumber;
                }
            });
        };
        loopAST(ast.children);
        this.syntaxLength = lineNumber;
    }
    #doCheck() {
        if (this.rawLength && this.syntaxLength) {
            if (this.rawLength !== this.syntaxLength) {
                console.warn("the rawLength not match for the syntaxLength");
            }
            Object.values(this.syntaxFile).forEach(({ value, lineNumber }) => {
                if (value !== this.rawFile[lineNumber]) {
                    console.log("some line not match:" + value + " __ " + this.rawFile[lineNumber] + " __ at: " + lineNumber + " lineNumber");
                }
            });
        }
    }
}
const getFile = (raw, lang, fileName) => {
    const key = raw + "--" + "0.0.6" + "--" + lang;
    if (map.has(key))
        return map.get(key);
    const file = new File(raw, lang, fileName);
    map.set(key, file);
    return file;
};

const maxLength = 1000;
/** Get the maximum position in the range. */
function rangeMax(range) {
    return range.location + range.length;
}
/** Get the length of the common substring between the two strings. */
function commonLength(stringA, rangeA, stringB, rangeB, reverse) {
    const max = Math.min(rangeA.length, rangeB.length);
    const startA = reverse ? rangeMax(rangeA) - 1 : rangeA.location;
    const startB = reverse ? rangeMax(rangeB) - 1 : rangeB.location;
    const stride = reverse ? -1 : 1;
    let length = 0;
    while (Math.abs(length) < max) {
        if (stringA[startA + length] !== stringB[startB + length]) {
            break;
        }
        length += stride;
    }
    return Math.abs(length);
}
function isInValidString(s) {
    return s.trim().length === 0 || s.length >= maxLength;
}
/** Get the changed ranges in the strings, relative to each other. */
function relativeChanges(stringA, stringB) {
    let bRange = { location: 0, length: stringB.length };
    let aRange = { location: 0, length: stringA.length };
    const _stringA = stringA.trimEnd();
    const _stringB = stringB.trimEnd();
    const aEndStr = stringA.slice(-2);
    const bEndStr = stringB.slice(-2);
    if (_stringA === _stringB && aEndStr !== bEndStr && (aEndStr === "\r\n" || bEndStr === "\r\n")) {
        return {
            stringARange: {
                location: _stringA.length,
                length: stringA.length - _stringA.length,
                isNewLineSymbolChanged: true,
            },
            stringBRange: {
                location: _stringB.length,
                length: stringB.length - _stringB.length,
                isNewLineSymbolChanged: true,
            },
        };
    }
    if (isInValidString(stringA) || isInValidString(stringB)) {
        aRange.length = 0;
        bRange.length = 0;
        return { stringARange: aRange, stringBRange: bRange };
    }
    const prefixLength = commonLength(stringB, bRange, stringA, aRange, false);
    bRange = {
        location: bRange.location + prefixLength,
        length: bRange.length - prefixLength,
    };
    aRange = {
        location: aRange.location + prefixLength,
        length: aRange.length - prefixLength,
    };
    const suffixLength = commonLength(stringB, bRange, stringA, aRange, true);
    bRange.length -= suffixLength;
    aRange.length -= suffixLength;
    return { stringARange: aRange, stringBRange: bRange };
}
/** Check two string have a diff range */
function hasRelativeChange(stringA, stringB) {
    const _stringA = stringA.trim();
    const _stringB = stringB.trim();
    const { stringARange, stringBRange } = relativeChanges(_stringA, _stringB);
    return (stringARange.location > 0 ||
        stringBRange.location > 0 ||
        stringARange.length < _stringA.length ||
        stringBRange.length < _stringB.length);
}

/** indicate what a line in the diff represents */
var DiffLineType;
(function (DiffLineType) {
    DiffLineType[DiffLineType["Context"] = 0] = "Context";
    DiffLineType[DiffLineType["Add"] = 1] = "Add";
    DiffLineType[DiffLineType["Delete"] = 2] = "Delete";
    DiffLineType[DiffLineType["Hunk"] = 3] = "Hunk";
})(DiffLineType || (DiffLineType = {}));
/** track details related to each line in the diff */
class DiffLine {
    text;
    type;
    originalLineNumber;
    oldLineNumber;
    newLineNumber;
    noTrailingNewLine;
    needRematch;
    range;
    constructor(text, type, 
    // Line number in the original diff patch (before expanding it), or null if
    // it was added as part of a diff expansion action.
    originalLineNumber, oldLineNumber, newLineNumber, noTrailingNewLine = false, needRematch = false, range) {
        this.text = text;
        this.type = type;
        this.originalLineNumber = originalLineNumber;
        this.oldLineNumber = oldLineNumber;
        this.newLineNumber = newLineNumber;
        this.noTrailingNewLine = noTrailingNewLine;
        this.needRematch = needRematch;
        this.range = range;
    }
    withNoTrailingNewLine(noTrailingNewLine) {
        return new DiffLine(this.text, this.type, this.originalLineNumber, this.oldLineNumber, this.newLineNumber, noTrailingNewLine);
    }
    isIncludeableLine() {
        return this.type === DiffLineType.Add || this.type === DiffLineType.Delete;
    }
    /** The content of the line, i.e., without the line type marker. */
    get content() {
        return this.text.substring(1);
    }
    equals(other) {
        return (this.text === other.text &&
            this.type === other.type &&
            this.originalLineNumber === other.originalLineNumber &&
            this.oldLineNumber === other.oldLineNumber &&
            this.newLineNumber === other.newLineNumber &&
            this.noTrailingNewLine === other.noTrailingNewLine);
    }
}
const checkDiffLineIncludeChange = (diffLine) => {
    if (!diffLine)
        return false;
    return diffLine.type === DiffLineType.Add || diffLine.type === DiffLineType.Delete;
};

var DiffHunkExpansionType;
(function (DiffHunkExpansionType) {
    /** The hunk header cannot be expanded at all. */
    DiffHunkExpansionType["None"] = "None";
    /**
     * The hunk header can be expanded up exclusively. Only the first hunk can be
     * expanded up exclusively.
     */
    DiffHunkExpansionType["Up"] = "Up";
    /**
     * The hunk header can be expanded down exclusively. Only the last hunk (if
     * it's the dummy hunk with only one line) can be expanded down exclusively.
     */
    DiffHunkExpansionType["Down"] = "Down";
    /** The hunk header can be expanded both up and down. */
    DiffHunkExpansionType["Both"] = "Both";
    /**
     * The hunk header represents a short gap that, when expanded, will
     * result in merging this hunk and the hunk above.
     */
    DiffHunkExpansionType["Short"] = "Short";
})(DiffHunkExpansionType || (DiffHunkExpansionType = {}));
/** each diff is made up of a number of hunks */
class DiffHunk {
    header;
    lines;
    unifiedDiffStart;
    unifiedDiffEnd;
    expansionType;
    /**
     * @param header The details from the diff hunk header about the line start and patch length.
     * @param lines The contents - context and changes - of the diff section.
     * @param unifiedDiffStart The diff hunk's start position in the overall file diff.
     * @param unifiedDiffEnd The diff hunk's end position in the overall file diff.
     */
    constructor(header, lines, unifiedDiffStart, unifiedDiffEnd, expansionType) {
        this.header = header;
        this.lines = lines;
        this.unifiedDiffStart = unifiedDiffStart;
        this.unifiedDiffEnd = unifiedDiffEnd;
        this.expansionType = expansionType;
    }
    equals(other) {
        if (this === other) {
            return true;
        }
        return (this.header.equals(other.header) &&
            this.unifiedDiffStart === other.unifiedDiffStart &&
            this.unifiedDiffEnd === other.unifiedDiffEnd &&
            this.expansionType === other.expansionType &&
            this.lines.length === other.lines.length &&
            this.lines.every((xLine, ix) => xLine.equals(other.lines[ix])));
    }
}
/** details about the start and end of a diff hunk */
class DiffHunkHeader {
    oldStartLine;
    oldLineCount;
    newStartLine;
    newLineCount;
    /**
     * @param oldStartLine The line in the old (or original) file where this diff hunk starts.
     * @param oldLineCount The number of lines in the old (or original) file that this diff hunk covers
     * @param newStartLine The line in the new file where this diff hunk starts.
     * @param newLineCount The number of lines in the new file that this diff hunk covers.
     */
    constructor(oldStartLine, oldLineCount, newStartLine, newLineCount) {
        this.oldStartLine = oldStartLine;
        this.oldLineCount = oldLineCount;
        this.newStartLine = newStartLine;
        this.newLineCount = newLineCount;
    }
    toDiffLineRepresentation() {
        return `@@ -${this.oldStartLine},${this.oldLineCount} +${this.newStartLine},${this.newLineCount} @@`;
    }
    equals(other) {
        return (this.oldStartLine === other.oldStartLine &&
            this.oldLineCount === other.oldLineCount &&
            this.newStartLine === other.newStartLine &&
            this.oldStartLine === other.oldStartLine);
    }
}

/** How many new lines will be added to a diff hunk by default. */
const DefaultDiffExpansionStep = 20;
function assertNever(_, message) {
    throw new Error(message);
}
/** Utility function for getting the digit count of the largest line number in an array of diff hunks */
function getLargestLineNumber(hunks) {
    if (hunks.length === 0) {
        return 0;
    }
    for (let i = hunks.length - 1; i >= 0; i--) {
        const hunk = hunks[i];
        for (let j = hunk.lines.length - 1; j >= 0; j--) {
            const line = hunk.lines[j];
            if (line.type === DiffLineType.Hunk) {
                continue;
            }
            const newLineNumber = line.newLineNumber ?? 0;
            const oldLineNumber = line.oldLineNumber ?? 0;
            return newLineNumber > oldLineNumber ? newLineNumber : oldLineNumber;
        }
    }
    return 0;
}
/**
 * Calculates whether or not a hunk header can be expanded up, down, both, or if
 * the space represented by the hunk header is short and expansion there would
 * mean merging with the hunk above.
 *
 * @param hunkIndex     Index of the hunk to evaluate within the whole diff.
 * @param hunkHeader    Header of the hunk to evaluate.
 * @param previousHunk  Hunk previous to the one to evaluate. Null if the
 *                      evaluated hunk is the first one.
 */
function getHunkHeaderExpansionType(hunkIndex, hunkHeader, previousHunk) {
    const distanceToPrevious = previousHunk === null
        ? Infinity
        : hunkHeader.oldStartLine - previousHunk.header.oldStartLine - previousHunk.header.oldLineCount;
    // In order to simplify the whole logic around expansion, only the hunk at the
    // top can be expanded up exclusively, and only the hunk at the bottom (the
    // dummy one, see getTextDiffWithBottomDummyHunk) can be expanded down
    // exclusively.
    // The rest of the hunks can be expanded both ways, except those which are too
    // short and therefore the direction of expansion doesn't matter.
    if (hunkIndex === 0) {
        // The top hunk can only be expanded if there is content above it
        if (hunkHeader.oldStartLine > 1 && hunkHeader.newStartLine > 1) {
            return DiffHunkExpansionType.Up;
        }
        else {
            return DiffHunkExpansionType.None;
        }
    }
    else if (distanceToPrevious <= DefaultDiffExpansionStep) {
        return DiffHunkExpansionType.Short;
    }
    else {
        return DiffHunkExpansionType.Both;
    }
}
const numIterator = (num, cb) => {
    const re = [];
    for (let i = 0; i < num; i++) {
        re.push(cb(i));
    }
    return re;
};
const getLang = (fileName) => {
    const dotIndex = fileName.lastIndexOf(".");
    const extension = fileName.slice(dotIndex + 1);
    return extension;
};
const getDiffRange = (additions, deletions) => {
    if (additions.length === deletions.length) {
        const len = additions.length;
        for (let i = 0; i < len; i++) {
            const addition = additions[i];
            const deletion = deletions[i];
            const hasDiffRange = hasRelativeChange(addition.text, deletion.text);
            if (hasDiffRange) {
                const { stringARange, stringBRange } = relativeChanges(addition.text, deletion.text);
                addition.needRematch = true;
                addition.range = stringARange;
                deletion.needRematch = true;
                deletion.range = stringBRange;
            }
        }
    }
};

/* eslint-disable max-lines */
// NODE: ALL of the Diff parse logic from desktop, SEE https://github.com/desktop/desktop
// https://en.wikipedia.org/wiki/Diff_utility
//
// @@ -l,s +l,s @@ optional section heading
//
// The hunk range information contains two hunk ranges. The range for the hunk of the original
// file is preceded by a minus symbol, and the range for the new file is preceded by a plus
// symbol. Each hunk range is of the format l,s where l is the starting line number and s is
// the number of lines the change hunk applies to for each respective file.
//
// In many versions of GNU diff, each range can omit the comma and trailing value s,
// in which case s defaults to 1
const diffHeaderRe = /^@@ -(\d+)(?:,(\d+))? \+(\d+)(?:,(\d+))? @@/;
/**
 * Regular expression matching invisible bidirectional Unicode characters that
 * may be interpreted or compiled differently than what it appears. More info:
 * https://github.co/hiddenchars
 */
const HiddenBidiCharsRegex = /[\u202A-\u202E]|[\u2066-\u2069]/;
const DiffPrefixAdd = "+";
const DiffPrefixDelete = "-";
const DiffPrefixContext = " ";
const DiffPrefixNoNewline = "\\";
const DiffLinePrefixChars = new Set([
    DiffPrefixAdd,
    DiffPrefixDelete,
    DiffPrefixContext,
    DiffPrefixNoNewline,
]);
/**
 * A parser for the GNU unified diff format
 *
 * See https://www.gnu.org/software/diffutils/manual/html_node/Detailed-Unified.html
 */
class DiffParser {
    /**
     * Line start pointer.
     *
     * The offset into the text property where the current line starts (ie either zero
     * or one character ahead of the last newline character).
     */
    ls;
    /**
     * Line end pointer.
     *
     * The offset into the text property where the current line ends (ie it points to
     * the newline character) or -1 if the line boundary hasn't been determined yet
     */
    le;
    /**
     * The text buffer containing the raw, unified diff output to be parsed
     */
    text;
    constructor() {
        Object.defineProperty(this, "__v_skip", { value: true });
        this.reset();
    }
    /**
     * Resets the internal parser state so that it can be reused.
     *
     * This is done automatically at the end of each parse run.
     */
    reset() {
        this.ls = 0;
        this.le = -1;
        this.text = "";
    }
    /**
     * Aligns the internal character pointers at the boundaries of
     * the next line.
     *
     * Returns true if successful or false if the end of the diff
     * has been reached.
     */
    nextLine() {
        this.ls = this.le + 1;
        // We've reached the end of the diff
        if (this.ls >= this.text.length) {
            return false;
        }
        this.le = this.text.indexOf("\n", this.ls);
        // If we can't find the next newline character we'll put our
        // end pointer at the end of the diff string
        if (this.le === -1) {
            this.le = this.text.length;
        }
        // We've succeeded if there's anything to read in between the
        // start and the end
        return this.ls !== this.le;
    }
    /**
     * Advances to the next line and returns it as a substring
     * of the raw diff text. Returns null if end of diff was
     * reached.
     */
    readLine(header) {
        if (header) {
            return this.nextLine() ? this.text.substring(this.ls, this.le) : null;
        }
        else {
            return this.nextLine() ? this.text.substring(this.ls + 1, this.le + 1) : null;
        }
    }
    /** Tests if the current line starts with the given search text */
    lineStartsWith(searchString) {
        return this.text.startsWith(searchString, this.ls);
    }
    /** Tests if the current line ends with the given search text */
    lineEndsWith(searchString) {
        return this.text.endsWith(searchString, this.le);
    }
    /**
     * Returns the starting character of the next line without
     * advancing the internal state. Returns null if advancing
     * would mean reaching the end of the diff.
     */
    peek() {
        const p = this.le + 1;
        return p < this.text.length ? this.text[p] : null;
    }
    /**
     * Parse the diff header, meaning everything from the
     * start of the diff output to the end of the line beginning
     * with +++
     *
     * Example diff header:
     *
     *   diff --git a/app/src/lib/diff-parser.ts b/app/src/lib/diff-parser.ts
     *   index e1d4871..3bd3ee0 100644
     *   --- a/app/src/lib/diff-parser.ts
     *   +++ b/app/src/lib/diff-parser.ts
     *
     * Returns an object with information extracted from the diff
     * header (currently whether it's a binary patch) or null if
     * the end of the diff was reached before the +++ line could be
     * found (which is a valid state).
     */
    parseDiffHeader() {
        // TODO: There's information in here that we might want to
        // capture, such as mode changes
        while (this.nextLine()) {
            if (this.lineStartsWith("Binary files ") && this.lineEndsWith("differ")) {
                return { isBinary: true };
            }
            if (this.lineStartsWith("+++")) {
                return { isBinary: false };
            }
        }
        // It's not an error to not find the +++ line, see the
        // 'parses diff of empty file' test in diff-parser-tests.ts
        return null;
    }
    /**
     * Attempts to convert a RegExp capture group into a number.
     * If the group doesn't exist or wasn't captured the function
     * will return the value of the defaultValue parameter or throw
     * an error if no default value was provided. If the captured
     * string can't be converted to a number an error will be thrown.
     */
    numberFromGroup(m, group, defaultValue = null) {
        const str = m[group];
        if (!str) {
            if (!defaultValue) {
                throw new Error(`Group ${group} missing from regexp match and no defaultValue was provided`);
            }
            return defaultValue;
        }
        const num = parseInt(str, 10);
        if (isNaN(num)) {
            throw new Error(`Could not parse capture group ${group} into number: ${str}`);
        }
        return num;
    }
    /**
     * Parses a hunk header or throws an error if the given line isn't
     * a well-formed hunk header.
     *
     * We currently only extract the line number information and
     * ignore any hunk headings.
     *
     * Example hunk header (text within ``):
     *
     * `@@ -84,10 +82,8 @@ export function parseRawDiff(lines: ReadonlyArray<string>): Diff {`
     *
     * Where everything after the last @@ is what's known as the hunk, or section, heading
     */
    parseHunkHeader(line) {
        const m = diffHeaderRe.exec(line);
        if (!m) {
            throw new Error(`Invalid hunk header format`);
        }
        // If endLines are missing default to 1, see diffHeaderRe docs
        const oldStartLine = this.numberFromGroup(m, 1);
        const oldLineCount = this.numberFromGroup(m, 2, 1);
        const newStartLine = this.numberFromGroup(m, 3);
        const newLineCount = this.numberFromGroup(m, 4, 1);
        return new DiffHunkHeader(oldStartLine, oldLineCount, newStartLine, newLineCount);
    }
    /**
     * Convenience function which lets us leverage the type system to
     * prove exhaustive checks in parseHunk.
     *
     * Takes an arbitrary string and checks to see if the first character
     * of that string is one of the allowed prefix characters for diff
     * lines (ie lines in between hunk headers).
     */
    parseLinePrefix(c) {
        // Since we know that DiffLinePrefixChars and the DiffLinePrefix type
        // include the same characters we can tell the type system that we
        // now know that c[0] is one of the characters in the DifflinePrefix set
        if (c && c.length && DiffLinePrefixChars.has(c[0])) {
            return c[0];
        }
        return null;
    }
    /**
     * Parses a hunk, including its header or throws an error if the diff doesn't
     * contain a well-formed diff hunk at the current position.
     *
     * Expects that the position has been advanced to the beginning of a presumed
     * diff hunk header.
     *
     * @param linesConsumed The number of unified diff lines consumed up until
     *                      this point by the diff parser. Used to give the
     *                      position and length (in lines) of the parsed hunk
     *                      relative to the overall parsed diff. These numbers
     *                      have no real meaning in the context of a diff and
     *                      are only used to aid the app in line-selections.
     */
    parseHunk(linesConsumed, hunkIndex, previousHunk) {
        const headerLine = this.readLine(true);
        if (!headerLine) {
            throw new Error("Expected hunk header but reached end of diff");
        }
        const header = this.parseHunkHeader(headerLine);
        const lines = new Array();
        lines.push(new DiffLine(headerLine, DiffLineType.Hunk, 1, null, null));
        let c;
        let rollingDiffBeforeCounter = header.oldStartLine;
        let rollingDiffAfterCounter = header.newStartLine;
        let diffLineNumber = linesConsumed;
        while ((c = this.parseLinePrefix(this.peek()))) {
            const line = this.readLine(false);
            diffLineNumber++;
            if (!line) {
                throw new Error("Expected unified diff line but reached end of diff");
            }
            // A marker indicating that the last line in the original or the new file
            // is missing a trailing newline. In other words, the presence of this marker
            // means that the new and/or original file lacks a trailing newline.
            //
            // When we find it we have to look up the previous line and set the
            // noTrailingNewLine flag
            if (c === DiffPrefixNoNewline) {
                // See https://github.com/git/git/blob/21f862b498925194f8f1ebe8203b7a7df756555b/apply.c#L1725-L1732
                if (line.length < 12) {
                    throw new Error(`Expected "no newline at end of file" marker to be at least 12 bytes long`);
                }
                const previousLineIndex = lines.length - 1;
                const previousLine = lines[previousLineIndex];
                lines[previousLineIndex] = previousLine.withNoTrailingNewLine(true);
                continue;
            }
            let diffLine;
            if (c === DiffPrefixAdd) {
                diffLine = new DiffLine(line, DiffLineType.Add, diffLineNumber, null, rollingDiffAfterCounter++);
            }
            else if (c === DiffPrefixDelete) {
                diffLine = new DiffLine(line, DiffLineType.Delete, diffLineNumber, rollingDiffBeforeCounter++, null);
            }
            else if (c === DiffPrefixContext) {
                diffLine = new DiffLine(line, DiffLineType.Context, diffLineNumber, rollingDiffBeforeCounter++, rollingDiffAfterCounter++);
            }
            else {
                return assertNever(c, `Unknown DiffLinePrefix: ${c}`);
            }
            lines.push(diffLine);
        }
        if (lines.length === 1) {
            throw new Error("Malformed diff, empty hunk");
        }
        return new DiffHunk(header, lines, linesConsumed, linesConsumed + lines.length - 1, getHunkHeaderExpansionType(hunkIndex, header, previousHunk));
    }
    /**
     * Parse a well-formed unified diff into hunks and lines.
     *
     * @param text A unified diff produced by git diff, git log --patch
     *             or any other git plumbing command that produces unified
     *             diffs.
     */
    parse(text) {
        this.text = text;
        try {
            const headerInfo = this.parseDiffHeader();
            const headerEnd = this.le;
            const header = this.text.substring(0, headerEnd);
            // empty diff
            if (!headerInfo) {
                return {
                    header,
                    contents: "",
                    hunks: [],
                    isBinary: false,
                    maxLineNumber: 0,
                    hasHiddenBidiChars: false,
                };
            }
            if (headerInfo.isBinary) {
                return {
                    header,
                    contents: "",
                    hunks: [],
                    isBinary: true,
                    maxLineNumber: 0,
                    hasHiddenBidiChars: false,
                };
            }
            const hunks = new Array();
            let linesConsumed = 0;
            let previousHunk = null;
            do {
                const hunk = this.parseHunk(linesConsumed, hunks.length, previousHunk);
                hunks.push(hunk);
                previousHunk = hunk;
                linesConsumed += hunk.lines.length;
            } while (this.peek());
            const contents = this.text
                .substring(headerEnd + 1, this.le)
                // Note that this simply returns a reference to the
                // substring if no match is found, it does not create
                // a new string instance.
                .replace(/\n\\ No newline at end of file/g, "");
            return {
                header,
                contents,
                hunks,
                isBinary: headerInfo.isBinary,
                maxLineNumber: getLargestLineNumber(hunks),
                hasHiddenBidiChars: HiddenBidiCharsRegex.test(text),
            };
        }
        finally {
            this.reset();
        }
    }
}
const parseInstance = new DiffParser();

/* eslint-disable @typescript-eslint/ban-ts-comment */
/* eslint-disable max-lines */
const composeLen = 40;
const idSet = new Set();
class DiffFile {
    _oldFileName;
    _newFileName;
    _diffList;
    #oldFileResult;
    #newFileResult;
    #diffListResults;
    #diffLines;
    #oldFileDiffLines;
    #newFileDiffLines;
    #oldFileLines;
    #newFileLines;
    #oldFileSyntaxLines;
    #newFileSyntaxLines;
    #splitLeftLines = [];
    #splitRightLines = [];
    #splitHunksLines;
    #unifiedLines = [];
    #unifiedHunksLines;
    #listeners = [];
    #hasInitRaw = false;
    #hasInitSyntax = false;
    #hasBuildSplit = false;
    #hasBuildUnified = false;
    #updateCount = 0;
    #composeByDiff = false;
    _version_ = "0.0.6";
    _oldFileContent = "";
    _oldFileLang = "";
    _newFileContent = "";
    _newFileLang = "";
    diffLineLength = 0;
    splitLineLength = 0;
    unifiedLineLength = 0;
    #id = "";
    #clonedInstance = new Map();
    static createInstance(data, bundle) {
        const instance = new DiffFile(data?.oldFile?.fileName || "", data?.oldFile?.content || "", data?.newFile?.fileName || "", data?.newFile?.content || "", data?.hunks || [], data?.oldFile?.fileLang || "", data?.newFile?.fileLang || "");
        if (bundle) {
            instance.mergeBundle(bundle);
        }
        return instance;
    }
    constructor(_oldFileName, _oldFileContent, _newFileName, _newFileContent, _diffList, _oldFileLang, _newFileLang) {
        this._oldFileName = _oldFileName;
        this._newFileName = _newFileName;
        this._diffList = _diffList;
        Object.defineProperty(this, "__v_skip", { value: true });
        let oldContent = _oldFileContent;
        let newContent = _newFileContent;
        const diffList = Array.from(new Set(_diffList));
        Object.defineProperties(this, {
            _oldFileName: { get: () => _oldFileName },
            _newFileName: { get: () => _newFileName },
            _oldFileLang: { get: () => getLang(_oldFileLang || _oldFileName || _newFileLang || _newFileName) || "txt" },
            _newFileLang: { get: () => getLang(_newFileLang || _newFileName || _oldFileLang || _oldFileName) || "txt" },
            _oldFileContent: {
                get: () => oldContent,
                set: (v) => (oldContent = v),
            },
            _newFileContent: {
                get: () => newContent,
                set: (v) => (newContent = v),
            },
            _diffList: { get: () => diffList },
        });
        this.initId();
    }
    #doDiff() {
        if (!this._diffList)
            return;
        this.#diffListResults = this._diffList.map((s) => parseInstance.parse(s));
    }
    #doFile() {
        if (!this._oldFileContent && !this._newFileContent)
            return;
        if (this._oldFileContent) {
            this.#oldFileResult = getFile(this._oldFileContent, this._oldFileLang, this._oldFileName);
        }
        if (this._newFileContent) {
            this.#newFileResult = getFile(this._newFileContent, this._newFileLang, this._newFileName);
        }
    }
    #composeRaw() {
        this.#oldFileResult?.doRaw();
        this.#oldFileLines = this.#oldFileResult?.rawFile;
        this.#newFileResult?.doRaw();
        this.#newFileLines = this.#newFileResult?.rawFile;
    }
    #composeFile() {
        if (this._oldFileContent && this._newFileContent)
            return;
        // all of the file content not exist, try to use diff result to compose
        if (!this._oldFileContent && !this._newFileContent) {
            let newLineNumber = 1;
            let oldLineNumber = 1;
            let oldFileContent = "";
            let newFileContent = "";
            while (oldLineNumber <= this.diffLineLength) {
                const diffLine = this.#getOldDiffLine(oldLineNumber++);
                if (diffLine) {
                    oldFileContent += diffLine.text;
                }
                else {
                    // empty line for placeholder
                    oldFileContent += "\n";
                }
            }
            while (newLineNumber <= this.diffLineLength) {
                const diffLine = this.#getNewDiffLine(newLineNumber++);
                if (diffLine) {
                    newFileContent += diffLine.text;
                }
                else {
                    // empty line for placeholder
                    newFileContent += "\n";
                }
            }
            if (oldFileContent === newFileContent)
                return;
            this._oldFileContent = oldFileContent;
            this._newFileContent = newFileContent;
            this.#oldFileResult = getFile(this._oldFileContent, this._oldFileLang, this._oldFileName);
            this.#newFileResult = getFile(this._newFileContent, this._newFileLang, this._newFileName);
            // all of the file just compose by diff, so we can not do the expand action
            this.#composeByDiff = true;
        }
        else if (this.#oldFileResult) {
            let newLineNumber = 1;
            let oldLineNumber = 1;
            let newFileContent = "";
            while (oldLineNumber <= this.#oldFileResult.maxLineNumber) {
                const newDiffLine = this.#getNewDiffLine(newLineNumber++);
                if (newDiffLine) {
                    newFileContent += newDiffLine.text;
                    oldLineNumber = newDiffLine.oldLineNumber ? newDiffLine.oldLineNumber + 1 : oldLineNumber;
                }
                else {
                    newFileContent += this.#getOldRawLine(oldLineNumber++);
                }
            }
            if (newFileContent === this._oldFileContent)
                return;
            this._newFileContent = newFileContent;
            this.#newFileResult = getFile(this._newFileContent, this._newFileLang, this._newFileName);
        }
        else if (this.#newFileResult) {
            let oldLineNumber = 1;
            let newLineNumber = 1;
            let oldFileContent = "";
            while (newLineNumber <= this.#newFileResult.maxLineNumber) {
                const oldDiffLine = this.#getOldDiffLine(oldLineNumber++);
                if (oldDiffLine) {
                    oldFileContent += oldDiffLine.text;
                    newLineNumber = oldDiffLine.newLineNumber ? oldDiffLine.newLineNumber + 1 : newLineNumber;
                }
                else {
                    oldFileContent += this.#getNewRawLine(newLineNumber++);
                }
            }
            if (oldFileContent === this._newFileContent)
                return;
            this._oldFileContent = oldFileContent;
            this.#oldFileResult = getFile(this._oldFileContent, this._oldFileLang, this._oldFileName);
        }
        this.#composeRaw();
    }
    #composeDiff() {
        if (!this.#diffListResults?.length)
            return;
        this.#diffListResults.forEach((item) => {
            const hunks = item.hunks;
            hunks.forEach((hunk) => {
                let additions = [];
                let deletions = [];
                hunk.lines.forEach((line) => {
                    if (line.type === DiffLineType.Add) {
                        additions.push(line);
                    }
                    else if (line.type === DiffLineType.Delete) {
                        deletions.push(line);
                    }
                    else {
                        getDiffRange(additions, deletions);
                        additions = [];
                        deletions = [];
                    }
                });
                getDiffRange(additions, deletions);
            });
        });
        this.#diffLines = [];
        const tmp = [];
        this.#diffListResults.forEach((item) => {
            item.hunks.forEach((_item) => {
                tmp.push(..._item.lines);
            });
        });
        this.#diffLines = tmp.map((i, index) => {
            const typedI = i;
            typedI.index = index;
            if (typedI.type === DiffLineType.Hunk) {
                const numInfo = typedI.text.split("@@")?.[1].split(" ").filter(Boolean);
                const oldNumInfo = numInfo?.[0] || "";
                const newNumInfo = numInfo?.[1] || "";
                const [oldNumStartIndex, oldNumLength] = oldNumInfo.split(",");
                const [newNumStartIndex, newNumLength] = newNumInfo.split(",");
                const typedTypeI = typedI;
                typedTypeI.hunkInfo = {
                    oldStartIndex: -Number(oldNumStartIndex),
                    oldLength: Number(oldNumLength),
                    newStartIndex: +Number(newNumStartIndex),
                    newLength: Number(newNumLength),
                    _oldStartIndex: -Number(oldNumStartIndex),
                    _oldLength: Number(oldNumLength),
                    _newStartIndex: +Number(newNumStartIndex),
                    _newLength: Number(newNumLength),
                };
            }
            return typedI;
        });
        // this.#diffLines = this.#diffListResults
        //   .reduce<DiffLine[]>((p, c) => p.concat(...c.hunks.reduce<DiffLine[]>((_p, _c) => _p.concat(..._c.lines), [])), [])
        //   .map<DiffLineItem>((i, index) => {
        //     const typedI = i as DiffLineItem;
        //     typedI.index = index;
        //     if (typedI.type === DiffLineType.Hunk) {
        //       const numInfo = typedI.text.split("@@")?.[1].split(" ").filter(Boolean);
        //       const oldNumInfo = numInfo?.[0] || "";
        //       const newNumInfo = numInfo?.[1] || "";
        //       const [oldNumStartIndex, oldNumLength] = oldNumInfo.split(",");
        //       const [newNumStartIndex, newNumLength] = newNumInfo.split(",");
        //       const typedTypeI = typedI as DiffHunkItem;
        //       typedTypeI.hunkInfo = {
        //         oldStartIndex: -Number(oldNumStartIndex),
        //         oldLength: Number(oldNumLength),
        //         newStartIndex: +Number(newNumStartIndex),
        //         newLength: Number(newNumLength),
        //       };
        //     }
        //     return typedI;
        //   });
        this.#oldFileDiffLines = {};
        this.#diffLines.forEach((item) => {
            if (item.oldLineNumber) {
                this.diffLineLength = Math.max(this.diffLineLength, item.oldLineNumber);
                this.#oldFileDiffLines[item.oldLineNumber] = item;
            }
        });
        this.#newFileDiffLines = {};
        this.#diffLines.forEach((item) => {
            if (item.newLineNumber) {
                this.diffLineLength = Math.max(this.diffLineLength, item.newLineNumber);
                this.#newFileDiffLines[item.newLineNumber] = item;
            }
        });
    }
    #composeSyntax({ autoDetectLang, registerHighlighter, }) {
        this.#oldFileResult?.doSyntax({ autoDetectLang, registerHighlighter });
        this.#oldFileSyntaxLines = this.#oldFileResult?.syntaxFile;
        this.#newFileResult?.doSyntax({ autoDetectLang, registerHighlighter });
        this.#newFileSyntaxLines = this.#newFileResult?.syntaxFile;
    }
    #getOldDiffLine(lineNumber) {
        if (!lineNumber)
            return;
        return this.#oldFileDiffLines?.[lineNumber];
    }
    #getNewDiffLine(lineNumber) {
        if (!lineNumber)
            return;
        return this.#newFileDiffLines?.[lineNumber];
    }
    #getOldRawLine(lineNumber) {
        return this.#oldFileLines?.[lineNumber];
    }
    #getNewRawLine(lineNumber) {
        return this.#newFileLines?.[lineNumber];
    }
    initId() {
        let id = "--" + Math.random().toString().slice(2);
        while (idSet.has(id)) {
            id = "--" + Math.random().toString().slice(2);
        }
        idSet.add(id);
        this.#id = id;
    }
    getId() {
        return this.#id;
    }
    clearId() {
        idSet.delete(this.#id);
    }
    initRaw() {
        if (this.#hasInitRaw)
            return;
        this.#doDiff();
        this.#composeDiff();
        this.#doFile();
        this.#composeRaw();
        this.#composeFile();
        this.#hasInitRaw = true;
    }
    initSyntax({ autoDetectLang, registerHighlighter, } = {}) {
        if (this.#hasInitSyntax)
            return;
        this.#composeSyntax({ registerHighlighter, autoDetectLang });
        this.#hasInitSyntax = true;
    }
    init() {
        this.initRaw();
        this.initSyntax();
    }
    buildSplitDiffLines() {
        if (this.#hasBuildSplit)
            return;
        let oldFileLineNumber = 1;
        let newFileLineNumber = 1;
        let prevIsHidden = false;
        let hideStart = Infinity;
        const maxOldFileLineNumber = this.#oldFileResult?.maxLineNumber || 0;
        const maxNewFileLineNumber = this.#newFileResult?.maxLineNumber || 0;
        while (oldFileLineNumber <= maxOldFileLineNumber || newFileLineNumber <= maxNewFileLineNumber) {
            const oldDiffLine = this.#getOldDiffLine(oldFileLineNumber);
            const newDiffLine = this.#getNewDiffLine(newFileLineNumber);
            const oldRawLine = this.#getOldRawLine(oldFileLineNumber);
            const newRawLine = this.#getNewRawLine(newFileLineNumber);
            const oldLineHasChange = oldDiffLine?.isIncludeableLine();
            const newLineHasChange = newDiffLine?.isIncludeableLine();
            const len = this.#splitRightLines.length;
            const isHidden = !oldDiffLine && !newDiffLine;
            if ((oldDiffLine && !newDiffLine && oldDiffLine.newLineNumber && oldDiffLine.newLineNumber > newFileLineNumber) ||
                (!oldDiffLine && newDiffLine && newDiffLine.oldLineNumber && newDiffLine.oldLineNumber > oldFileLineNumber)) {
                if (this.#composeByDiff) {
                    oldDiffLine && newFileLineNumber++;
                    newDiffLine && oldFileLineNumber++;
                    continue;
                }
                else {
                    if (process.env.NODE_ENV === "development") {
                        console.error("some error happen for generate diff line!");
                    }
                }
            }
            if (!oldDiffLine && !newRawLine && !oldDiffLine && !newDiffLine)
                break;
            if ((oldLineHasChange && newLineHasChange) || (!oldLineHasChange && !newLineHasChange)) {
                this.#splitLeftLines.push({
                    lineNumber: oldFileLineNumber++,
                    value: oldRawLine,
                    diff: oldDiffLine,
                    isHidden,
                    _isHidden: isHidden,
                });
                this.#splitRightLines.push({
                    lineNumber: newFileLineNumber++,
                    value: newRawLine,
                    diff: newDiffLine,
                    isHidden,
                    _isHidden: isHidden,
                });
            }
            else if (oldLineHasChange) {
                this.#splitLeftLines.push({
                    lineNumber: oldFileLineNumber++,
                    value: oldRawLine,
                    diff: oldDiffLine,
                    isHidden,
                    _isHidden: isHidden,
                });
                this.#splitRightLines.push({});
            }
            else if (newLineHasChange) {
                this.#splitLeftLines.push({});
                this.#splitRightLines.push({
                    lineNumber: newFileLineNumber++,
                    value: newRawLine,
                    diff: newDiffLine,
                    isHidden,
                    _isHidden: isHidden,
                });
            }
            if (!prevIsHidden && isHidden) {
                hideStart = len;
            }
            prevIsHidden = isHidden;
            if (oldDiffLine && newDiffLine && !oldLineHasChange && !newLineHasChange) {
                const current = newDiffLine;
                const previous = newDiffLine.index ? this.#diffLines?.[current.index - 1] : undefined;
                if (previous && previous.type === DiffLineType.Hunk) {
                    const typedPrevious = previous;
                    if (Number.isFinite(hideStart)) {
                        typedPrevious.splitInfo = {
                            ...typedPrevious.hunkInfo,
                            startHiddenIndex: hideStart,
                            endHiddenIndex: len,
                            plainText: typedPrevious.text,
                            _startHiddenIndex: hideStart,
                            _endHiddenIndex: len,
                            _plainText: typedPrevious.text,
                        };
                        hideStart = Infinity;
                    }
                    this.#splitHunksLines = {
                        ...this.#splitHunksLines,
                        [len]: typedPrevious,
                    };
                }
            }
        }
        // have last hunk
        if (Number.isFinite(hideStart)) {
            const lastDiff = new DiffLine("", DiffLineType.Hunk, null, null, null);
            const lastHunk = lastDiff;
            lastHunk.isLast = true;
            lastHunk.splitInfo = {
                startHiddenIndex: hideStart,
                endHiddenIndex: this.#splitRightLines.length,
                _startHiddenIndex: hideStart,
                _endHiddenIndex: this.#splitRightLines.length,
                // just for placeholder
                plainText: "",
                oldStartIndex: 0,
                newStartIndex: 0,
                oldLength: 0,
                newLength: 0,
                _plainText: "",
                _oldStartIndex: 0,
                _newStartIndex: 0,
                _oldLength: 0,
                _newLength: 0,
            };
            this.#splitHunksLines = {
                ...this.#splitHunksLines,
                [this.#splitRightLines.length]: lastHunk,
            };
            hideStart = Infinity;
        }
        this.splitLineLength = this.#splitRightLines.length;
        this.#hasBuildSplit = true;
        this.notifyAll();
    }
    buildUnifiedDiffLines() {
        if (this.#hasBuildUnified)
            return;
        let oldFileLineNumber = 1;
        let newFileLineNumber = 1;
        let prevIsHidden = false;
        let hideStart = Infinity;
        const maxOldFileLineNumber = this.#oldFileResult?.maxLineNumber || 0;
        const maxNewFileLineNumber = this.#newFileResult?.maxLineNumber || 0;
        while (oldFileLineNumber <= maxOldFileLineNumber || newFileLineNumber <= maxNewFileLineNumber) {
            const oldRawLine = this.#getOldRawLine(oldFileLineNumber);
            const oldDiffLine = this.#getOldDiffLine(oldFileLineNumber);
            const newRawLine = this.#getNewRawLine(newFileLineNumber);
            const newDiffLine = this.#getNewDiffLine(newFileLineNumber);
            const oldLineHasChange = oldDiffLine?.isIncludeableLine();
            const newLineHasChange = newDiffLine?.isIncludeableLine();
            const len = this.#unifiedLines.length;
            const isHidden = !oldDiffLine && !newDiffLine;
            if ((oldDiffLine && !newDiffLine && oldDiffLine.newLineNumber && oldDiffLine.newLineNumber > newFileLineNumber) ||
                (!oldDiffLine && newDiffLine && newDiffLine.oldLineNumber && newDiffLine.oldLineNumber > oldFileLineNumber)) {
                if (this.#composeByDiff) {
                    oldDiffLine && newFileLineNumber++;
                    newDiffLine && oldFileLineNumber++;
                    continue;
                }
                else {
                    if (process.env.NODE_ENV === "development") {
                        console.error("some error happen for generate diff line!");
                    }
                }
            }
            if (!oldRawLine && !newRawLine && !newDiffLine && !oldDiffLine)
                break;
            if (!oldLineHasChange && !newLineHasChange) {
                this.#unifiedLines.push({
                    oldLineNumber: oldFileLineNumber++,
                    newLineNumber: newFileLineNumber++,
                    value: newRawLine,
                    diff: newDiffLine,
                    isHidden,
                    _isHidden: isHidden,
                });
            }
            else if (oldLineHasChange) {
                this.#unifiedLines.push({
                    oldLineNumber: oldFileLineNumber++,
                    value: oldRawLine,
                    diff: oldDiffLine,
                    isHidden,
                    _isHidden: isHidden,
                });
            }
            else if (newLineHasChange) {
                this.#unifiedLines.push({
                    newLineNumber: newFileLineNumber++,
                    value: newRawLine,
                    diff: newDiffLine,
                    isHidden,
                    _isHidden: isHidden,
                });
            }
            if (!prevIsHidden && isHidden) {
                hideStart = len;
            }
            prevIsHidden = isHidden;
            if (oldDiffLine && newDiffLine && !oldLineHasChange && !newLineHasChange) {
                const current = newDiffLine;
                const previous = current.index ? this.#diffLines?.[current.index - 1] : undefined;
                if (previous && previous.type === DiffLineType.Hunk) {
                    const typedPrevious = previous;
                    if (Number.isFinite(hideStart)) {
                        typedPrevious.unifiedInfo = {
                            ...typedPrevious.hunkInfo,
                            startHiddenIndex: hideStart,
                            endHiddenIndex: len,
                            plainText: typedPrevious.text,
                            _startHiddenIndex: hideStart,
                            _endHiddenIndex: len,
                            _plainText: typedPrevious.text,
                        };
                        hideStart = Infinity;
                    }
                    this.#unifiedHunksLines = {
                        ...this.#unifiedHunksLines,
                        [len]: typedPrevious,
                    };
                }
            }
        }
        // have last hunk
        if (Number.isFinite(hideStart)) {
            const lastDiff = new DiffLine("", DiffLineType.Hunk, null, null, null);
            const lastHunk = lastDiff;
            lastHunk.isLast = true;
            lastHunk.unifiedInfo = {
                startHiddenIndex: hideStart,
                endHiddenIndex: this.#unifiedLines.length,
                _startHiddenIndex: hideStart,
                _endHiddenIndex: this.#unifiedLines.length,
                // just for placeholder
                plainText: "",
                oldStartIndex: 0,
                newStartIndex: 0,
                oldLength: 0,
                newLength: 0,
                _plainText: "",
                _oldStartIndex: 0,
                _newStartIndex: 0,
                _oldLength: 0,
                _newLength: 0,
            };
            this.#unifiedHunksLines = {
                ...this.#unifiedHunksLines,
                [this.#unifiedLines.length]: lastHunk,
            };
            hideStart = Infinity;
        }
        this.unifiedLineLength = this.#unifiedLines.length;
        this.#hasBuildUnified = true;
        this.notifyAll();
    }
    getSplitLeftLine = (index) => {
        return this.#splitLeftLines[index];
    };
    getSplitRightLine = (index) => {
        return this.#splitRightLines[index];
    };
    getSplitHunkLine = (index) => {
        return this.#splitHunksLines?.[index];
    };
    onSplitHunkExpand = (dir, index, needTrigger = true) => {
        const current = this.#splitHunksLines?.[index];
        if (!current || !current.splitInfo)
            return;
        if (dir === "all") {
            for (let i = current.splitInfo.startHiddenIndex; i < current.splitInfo.endHiddenIndex; i++) {
                const leftLine = this.#splitLeftLines[i];
                const rightLine = this.#splitRightLines[i];
                if (leftLine?.isHidden)
                    leftLine.isHidden = false;
                if (rightLine?.isHidden)
                    rightLine.isHidden = false;
            }
            current.splitInfo = {
                ...current.splitInfo,
                ...current.hunkInfo,
                plainText: current.text,
                startHiddenIndex: current.splitInfo.endHiddenIndex,
            };
        }
        else if (dir === "down") {
            for (let i = current.splitInfo.startHiddenIndex; i < current.splitInfo.startHiddenIndex + composeLen; i++) {
                const leftLine = this.#splitLeftLines[i];
                const rightLine = this.#splitRightLines[i];
                if (leftLine?.isHidden)
                    leftLine.isHidden = false;
                if (rightLine?.isHidden)
                    rightLine.isHidden = false;
            }
            if (current.isLast) {
                current.splitInfo = {
                    ...current.splitInfo,
                    startHiddenIndex: current.splitInfo.startHiddenIndex + composeLen,
                };
            }
            else {
                current.splitInfo = {
                    ...current.splitInfo,
                    startHiddenIndex: current.splitInfo.startHiddenIndex + composeLen,
                    plainText: `@@ -${current.splitInfo.oldStartIndex},${current.splitInfo.oldLength} +${current.splitInfo.newStartIndex},${current.splitInfo.newLength}`,
                };
            }
        }
        else {
            if (current.isLast) {
                process.env.NODE_ENV === "development" && console.error("the last hunk can not expand up!");
                return;
            }
            for (let i = current.splitInfo.endHiddenIndex - composeLen; i < current.splitInfo.endHiddenIndex; i++) {
                const leftLine = this.#splitLeftLines[i];
                const rightLine = this.#splitRightLines[i];
                if (leftLine?.isHidden)
                    leftLine.isHidden = false;
                if (rightLine?.isHidden)
                    rightLine.isHidden = false;
            }
            const oldStartIndex = current.splitInfo.oldStartIndex - composeLen;
            const oldLength = current.splitInfo.oldLength + composeLen;
            const newStartIndex = current.splitInfo.newStartIndex - composeLen;
            const newLength = current.splitInfo.newLength + composeLen;
            current.splitInfo = {
                ...current.splitInfo,
                endHiddenIndex: current.splitInfo.endHiddenIndex - composeLen,
                oldStartIndex,
                oldLength,
                newStartIndex,
                newLength,
                plainText: `@@ -${oldStartIndex},${oldLength} +${newStartIndex},${newLength}`,
            };
            delete this.#splitHunksLines?.[index];
            this.#splitHunksLines[current.splitInfo.endHiddenIndex] = current;
        }
        needTrigger && this.notifyAll();
    };
    getUnifiedLine = (index) => {
        return this.#unifiedLines[index];
    };
    getUnifiedHunkLine = (index) => {
        return this.#unifiedHunksLines?.[index];
    };
    onUnifiedHunkExpand = (dir, index, needTrigger = true) => {
        const current = this.#unifiedHunksLines?.[index];
        if (!current || !current.unifiedInfo)
            return;
        if (dir === "all") {
            for (let i = current.unifiedInfo.startHiddenIndex; i < current.unifiedInfo.endHiddenIndex; i++) {
                const unifiedLine = this.#unifiedLines?.[i];
                if (unifiedLine?.isHidden) {
                    unifiedLine.isHidden = false;
                }
            }
            current.unifiedInfo = {
                ...current.unifiedInfo,
                ...current.hunkInfo,
                plainText: current.text,
                startHiddenIndex: current.unifiedInfo.endHiddenIndex,
            };
        }
        else if (dir === "down") {
            for (let i = current.unifiedInfo.startHiddenIndex; i < current.unifiedInfo.startHiddenIndex + composeLen; i++) {
                const unifiedLine = this.#unifiedLines[i];
                if (unifiedLine?.isHidden)
                    unifiedLine.isHidden = false;
            }
            if (current.isLast) {
                current.unifiedInfo = {
                    ...current.unifiedInfo,
                    startHiddenIndex: current.unifiedInfo.startHiddenIndex + composeLen,
                };
            }
            else {
                current.unifiedInfo = {
                    ...current.unifiedInfo,
                    startHiddenIndex: current.unifiedInfo.startHiddenIndex + composeLen,
                    plainText: `@@ -${current.unifiedInfo.oldStartIndex},${current.unifiedInfo.oldLength} +${current.unifiedInfo.newStartIndex},${current.unifiedInfo.newLength}`,
                };
            }
        }
        else {
            if (current.isLast) {
                process.env.NODE_ENV === "development" && console.error("the last hunk can not expand up!");
                return;
            }
            for (let i = current.unifiedInfo.endHiddenIndex - composeLen; i < current.unifiedInfo.endHiddenIndex; i++) {
                const unifiedLine = this.#unifiedLines[i];
                if (unifiedLine?.isHidden)
                    unifiedLine.isHidden = false;
            }
            const oldStartIndex = current.unifiedInfo.oldStartIndex - composeLen;
            const oldLength = current.unifiedInfo.oldLength + composeLen;
            const newStartIndex = current.unifiedInfo.newStartIndex - composeLen;
            const newLength = current.unifiedInfo.newLength + composeLen;
            current.unifiedInfo = {
                ...current.unifiedInfo,
                endHiddenIndex: current.unifiedInfo.endHiddenIndex - composeLen,
                oldStartIndex,
                oldLength,
                newStartIndex,
                newLength,
                plainText: `@@ -${oldStartIndex},${oldLength} +${newStartIndex},${newLength}`,
            };
            delete this.#unifiedHunksLines?.[index];
            this.#unifiedHunksLines[current.unifiedInfo.endHiddenIndex] = current;
        }
        needTrigger && this.notifyAll();
    };
    onAllExpand = (mode) => {
        if (mode === "split") {
            Object.keys(this.#splitHunksLines || {}).forEach((key) => {
                this.onSplitHunkExpand("all", +key, false);
            });
        }
        else {
            Object.keys(this.#unifiedHunksLines || {}).forEach((key) => {
                this.onUnifiedHunkExpand("all", +key, false);
            });
        }
        this.notifyAll();
    };
    onAllCollapse = (mode) => {
        if (mode === "split") {
            Object.values(this.#splitLeftLines || {}).forEach((item) => {
                if (!item.isHidden && item._isHidden) {
                    item.isHidden = item._isHidden;
                }
            });
            Object.values(this.#splitRightLines || {}).forEach((item) => {
                if (!item.isHidden && item._isHidden) {
                    item.isHidden = item._isHidden;
                }
            });
            Object.values(this.#splitHunksLines || {}).forEach((item) => {
                if (!item.splitInfo)
                    return;
                item.splitInfo = {
                    ...item.splitInfo,
                    oldStartIndex: item.splitInfo._oldStartIndex,
                    oldLength: item.splitInfo._oldLength,
                    newStartIndex: item.splitInfo._newStartIndex,
                    newLength: item.splitInfo._newLength,
                    startHiddenIndex: item.splitInfo._startHiddenIndex,
                    endHiddenIndex: item.splitInfo._endHiddenIndex,
                    plainText: item.splitInfo._plainText,
                };
            });
            Object.keys(this.#splitHunksLines || {}).forEach((key) => {
                const item = this.#splitHunksLines[key];
                if (!item.splitInfo)
                    return;
                if (item.splitInfo.endHiddenIndex !== +key) {
                    delete this.#splitHunksLines[key];
                    this.#splitHunksLines[item.splitInfo.endHiddenIndex] = item;
                }
            });
        }
        else {
            Object.values(this.#unifiedLines || {}).forEach((item) => {
                if (!item.isHidden && item._isHidden) {
                    item.isHidden = item._isHidden;
                }
            });
            Object.values(this.#unifiedHunksLines || {}).forEach((item) => {
                if (!item.unifiedInfo)
                    return;
                item.unifiedInfo = {
                    ...item.unifiedInfo,
                    oldStartIndex: item.unifiedInfo._oldStartIndex,
                    oldLength: item.unifiedInfo._oldLength,
                    newStartIndex: item.unifiedInfo._newStartIndex,
                    newLength: item.unifiedInfo._newLength,
                    startHiddenIndex: item.unifiedInfo._startHiddenIndex,
                    endHiddenIndex: item.unifiedInfo._endHiddenIndex,
                    plainText: item.unifiedInfo._plainText,
                };
            });
            Object.keys(this.#unifiedHunksLines || {}).forEach((key) => {
                const item = this.#unifiedHunksLines[key];
                if (!item.unifiedInfo)
                    return;
                if (item.unifiedInfo.endHiddenIndex !== +key) {
                    delete this.#unifiedHunksLines[key];
                    this.#unifiedHunksLines[item.unifiedInfo.endHiddenIndex] = item;
                }
            });
        }
        this.notifyAll();
    };
    getOldSyntaxLine = (lineNumber) => {
        return this.#oldFileSyntaxLines?.[lineNumber];
    };
    getNewSyntaxLine = (lineNumber) => {
        return this.#newFileSyntaxLines?.[lineNumber];
    };
    subscribe = (listener) => {
        this.#listeners.push(listener);
        return () => {
            this.#listeners.filter((i) => i !== listener);
        };
    };
    notifyAll = (skipSyncExternal) => {
        this.#updateCount++;
        this.#listeners.forEach((f) => {
            if (skipSyncExternal && f.isSyncExternal) {
                return;
            }
            f();
        });
    };
    getUpdateCount = () => this.#updateCount;
    getExpandEnabled = () => !this.#composeByDiff;
    getBundle = () => {
        // common
        const hasInitRaw = this.#hasInitRaw;
        const hasInitSyntax = this.#hasInitSyntax;
        const hasBuildSplit = this.#hasBuildSplit;
        const hasBuildUnified = this.#hasBuildUnified;
        const oldFileLines = this.#oldFileLines;
        const oldFileDiffLines = this.#oldFileDiffLines;
        const oldFileSyntaxLines = this.#oldFileSyntaxLines;
        const newFileLines = this.#newFileLines;
        const newFileDiffLines = this.#newFileDiffLines;
        const newFileSyntaxLines = this.#newFileSyntaxLines;
        const splitLineLength = this.splitLineLength;
        const unifiedLineLength = this.unifiedLineLength;
        const composeByDiff = this.#composeByDiff;
        // split
        const splitLeftLines = this.#splitLeftLines;
        const splitRightLines = this.#splitRightLines;
        const splitHunkLines = this.#splitHunksLines;
        // unified
        const unifiedLines = this.#unifiedLines;
        const unifiedHunkLines = this.#unifiedHunksLines;
        const version = this._version_;
        return {
            hasInitRaw,
            hasInitSyntax,
            hasBuildSplit,
            hasBuildUnified,
            oldFileLines,
            oldFileDiffLines,
            oldFileSyntaxLines,
            newFileLines,
            newFileDiffLines,
            newFileSyntaxLines,
            splitLineLength,
            unifiedLineLength,
            splitLeftLines,
            splitRightLines,
            splitHunkLines,
            unifiedLines,
            unifiedHunkLines,
            composeByDiff,
            version
        };
    };
    mergeBundle = (data) => {
        this.#hasInitRaw = data.hasInitRaw;
        this.#hasInitSyntax = data.hasInitSyntax;
        this.#hasBuildSplit = data.hasBuildSplit;
        this.#hasBuildUnified = data.hasBuildUnified;
        this.#composeByDiff = data.composeByDiff;
        this.#oldFileLines = data.oldFileLines;
        this.#oldFileDiffLines = data.oldFileDiffLines;
        this.#oldFileSyntaxLines = data.oldFileSyntaxLines;
        this.#newFileLines = data.newFileLines;
        this.#newFileDiffLines = data.newFileDiffLines;
        this.#newFileSyntaxLines = data.newFileSyntaxLines;
        this.splitLineLength = data.splitLineLength;
        this.unifiedLineLength = data.unifiedLineLength;
        this.#splitLeftLines = data.splitLeftLines;
        this.#splitRightLines = data.splitRightLines;
        this.#splitHunksLines = data.splitHunkLines;
        this.#unifiedLines = data.unifiedLines;
        this.#unifiedHunksLines = data.unifiedHunkLines;
        if (process.env.NODE_ENV === "development" && this._version_ !== data.version) {
            console.error('the version of the `diffInstance` is not match, some error may happen!');
        }
        this.notifyAll();
    };
    _addClonedInstance = (instance) => {
        const updateFunc = () => {
            this._notifyOthers(instance);
        };
        updateFunc.isSyncExternal = true;
        const unsubscribe = instance.subscribe(updateFunc);
        this.#clonedInstance.set(instance, unsubscribe);
    };
    _notifyOthers = (instance) => {
        this.#clonedInstance.forEach((_, i) => {
            if (i !== instance) {
                i.notifyAll(true);
            }
        });
    };
    _delClonedInstance = (instance) => {
        const unsubscribe = this.#clonedInstance.get(instance);
        unsubscribe && unsubscribe();
        this.#clonedInstance.delete(instance);
    };
    _getFullBundle = () => {
        const bundle = this.getBundle();
        const oldFileResult = this.#oldFileResult;
        const newFileResult = this.#newFileResult;
        const diffLines = this.#diffLines;
        const diffListResults = this.#diffListResults;
        return {
            ...bundle,
            oldFileResult,
            newFileResult,
            diffLines,
            diffListResults,
        };
    };
    _mergeFullBundle = (data) => {
        this.mergeBundle(data);
        this.#oldFileResult = data.oldFileResult;
        this.#newFileResult = data.newFileResult;
        this.#diffLines = data.diffLines;
        this.#diffListResults = data.diffListResults;
    };
    _destroy = () => {
        this.clearId();
        this.#listeners.splice(0, this.#listeners.length);
        this.#clonedInstance.forEach((v) => v());
        this.#clonedInstance.clear();
    };
    clear = () => {
        this._destroy();
        this.#oldFileResult = null;
        this.#newFileResult = null;
        this.#diffLines = null;
        this.#diffListResults = null;
        this.#newFileDiffLines = null;
        this.#oldFileDiffLines = null;
        this.#newFileLines = null;
        this.#oldFileLines = null;
        this.#newFileSyntaxLines = null;
        this.#oldFileSyntaxLines = null;
        this.#splitHunksLines = null;
        this.#splitLeftLines = null;
        this.#splitRightLines = null;
        this.#unifiedHunksLines = null;
        this.#unifiedLines = null;
    };
}

var DiffFileLineType;
(function (DiffFileLineType) {
    DiffFileLineType[DiffFileLineType["hunk"] = 1] = "hunk";
    DiffFileLineType[DiffFileLineType["content"] = 2] = "content";
    DiffFileLineType[DiffFileLineType["widget"] = 3] = "widget";
    DiffFileLineType[DiffFileLineType["extend"] = 4] = "extend";
})(DiffFileLineType || (DiffFileLineType = {}));
const getSplitLines = (diffFile, options) => {
    const splitLineLength = diffFile.splitLineLength;
    const splitLines = [];
    numIterator(splitLineLength, (index) => {
        const hunkLine = diffFile.getSplitHunkLine(index);
        const splitLeftLine = diffFile.getSplitLeftLine(index);
        const splitRightLine = diffFile.getSplitRightLine(index);
        const widgetLine = options?.hasRenderWidget && options.widgetData?.[index + 1];
        const extendLine = options?.hasRenderExtend && options.extendData?.[index + 1];
        hunkLine &&
            hunkLine.splitInfo &&
            hunkLine.splitInfo.startHiddenIndex < hunkLine.splitInfo.endHiddenIndex &&
            splitLines.push({ type: DiffFileLineType.hunk, index, lineNumber: index + 1, hunkLine: hunkLine });
        !splitLeftLine?.isHidden &&
            !splitRightLine?.isHidden &&
            splitLines.push({
                type: DiffFileLineType.content,
                index,
                lineNumber: index + 1,
                splitLine: { left: splitLeftLine, right: splitRightLine },
            });
        widgetLine &&
            splitLines.push({ type: DiffFileLineType.widget, index, lineNumber: index + 1, widgetLine: widgetLine });
        extendLine &&
            splitLines.push({ type: DiffFileLineType.extend, index, lineNumber: index + 1, extendLine: extendLine });
    });
    return splitLines;
};
const getSplitContentLines = (diffFile) => {
    const lines = getSplitLines(diffFile, {});
    return lines.filter((line) => line.type === DiffFileLineType.content);
};
const getUnifiedLines = (diffFile, options) => {
    const unifiedLineLength = diffFile.unifiedLineLength;
    const unifiedLines = [];
    numIterator(unifiedLineLength, (index) => {
        const hunkLine = diffFile.getUnifiedHunkLine(index);
        const unifiedLine = diffFile.getUnifiedLine(index);
        const widgetLine = options?.hasRenderWidget && options.widgetData?.[index + 1];
        const extendLine = options?.hasRenderExtend && options.extendData?.[index + 1];
        hunkLine &&
            hunkLine.unifiedInfo &&
            hunkLine.unifiedInfo.startHiddenIndex < hunkLine.unifiedInfo.endHiddenIndex &&
            unifiedLines.push({ type: DiffFileLineType.hunk, index, lineNumber: index + 1, hunkLine: hunkLine });
        !unifiedLine.isHidden &&
            unifiedLines.push({ type: DiffFileLineType.content, index, lineNumber: index + 1, unifiedLine: unifiedLine });
        widgetLine &&
            unifiedLines.push({ type: DiffFileLineType.widget, index, lineNumber: index + 1, widgetLine: widgetLine });
        extendLine &&
            unifiedLines.push({ type: DiffFileLineType.extend, index, lineNumber: index + 1, extendLine: extendLine });
    });
    return unifiedLines;
};
const getUnifiedContentLine = (diffFile) => {
    const lines = getUnifiedLines(diffFile, {});
    return lines.filter((line) => line.type === DiffFileLineType.content);
};

const useUnmount = (cb, deps) => {
    const ref = useRef(cb);
    ref.current = cb;
    useEffect(() => ref.current, deps);
};

var DiffModeEnum;
(function (DiffModeEnum) {
    DiffModeEnum[DiffModeEnum["Split"] = 1] = "Split";
    DiffModeEnum[DiffModeEnum["Unified"] = 2] = "Unified";
})(DiffModeEnum || (DiffModeEnum = {}));
const DiffViewContext = createContext(null);
DiffViewContext.displayName = "DiffViewContext";
const useDiffViewContext = () => useContext(DiffViewContext);

const isClient = typeof window !== "undefined";
const useSafeLayout = isClient ? useLayoutEffect : useEffect;

let canvasCtx = null;
class TextMeasure {
    #key = "";
    #map = {};
    #getInstance() {
        canvasCtx = canvasCtx || document.createElement("canvas").getContext("2d");
        return canvasCtx;
    }
    measure(text, font) {
        const currentKey = `${font?.fontFamily}-${font?.fontStyle}-${font?.fontSize}-${text}`;
        if (this.#map[currentKey]) {
            return this.#map[currentKey];
        }
        const instance = this.#getInstance();
        if (font) {
            const currentFontKey = `${font.fontFamily}-${font.fontStyle}-${font.fontSize}`;
            if (this.#key !== currentFontKey) {
                this.#key = currentFontKey;
                instance.font = `${font.fontStyle || ""} ${font.fontSize || ""} ${font.fontFamily || ""}`;
            }
        }
        else {
            instance.font = "";
        }
        const textWidth = instance.measureText(text).width;
        return textWidth;
    }
}
const measureInstance = new TextMeasure();
const useTextWidth = ({ text, font, }) => {
    const [width, setWidth] = useState(0);
    useSafeLayout(() => {
        const width = measureInstance.measure(text, font);
        setWidth(width);
    }, [text, font]);
    return width;
};

const useDomWidth = ({ selector, enable }) => {
    const [width, setWidth] = useState(0);
    const { useDiffContext } = useDiffViewContext();
    const id = useDiffContext(useCallback((s) => s.id, []));
    useEffect(() => {
        if (enable) {
            const container = document.querySelector(`#diff-root${id}`);
            const wrapper = container?.querySelector(selector);
            if (!wrapper)
                return;
            const typedWrapper = wrapper;
            const cb = () => {
                const rect = wrapper?.getBoundingClientRect();
                setWidth(rect?.width ?? 0);
            };
            cb();
            const cleanCb = () => {
                typedWrapper.__observeCallback.delete(cb);
                if (typedWrapper.__observeCallback.size === 0) {
                    typedWrapper.__observeInstance?.disconnect();
                    typedWrapper.removeAttribute("data-observe");
                    delete typedWrapper.__observeCallback;
                    delete typedWrapper.__observeInstance;
                }
            };
            if (typedWrapper.__observeCallback) {
                typedWrapper.__observeCallback.add(cb);
                return () => cleanCb();
            }
            typedWrapper.__observeCallback = new Set();
            typedWrapper.__observeCallback.add(cb);
            const observer = new ResizeObserver(() => typedWrapper.__observeCallback.forEach((cb) => cb()));
            typedWrapper.__observeInstance = observer;
            observer.observe(typedWrapper);
            typedWrapper.setAttribute("data-observe", "height");
            return () => cleanCb();
        }
    }, [selector, enable, id]);
    return width;
};

const useSyncHeight = ({ selector, side, enable, }) => {
    const { useDiffContext } = useDiffViewContext();
    const id = useDiffContext(useCallback((s) => s.id, []));
    useSafeLayout(() => {
        if (enable) {
            const container = document.querySelector(`#diff-root${id}`);
            const elements = Array.from(container?.querySelectorAll(selector) || []);
            if (elements.length === 2) {
                const ele1 = elements[0];
                const ele2 = elements[1];
                const cb = () => {
                    ele1.style.height = "auto";
                    ele2.style.height = "auto";
                    const rect1 = ele1.getBoundingClientRect();
                    const rect2 = ele2.getBoundingClientRect();
                    if (rect1.height !== rect2.height) {
                        const maxHeight = Math.max(rect1.height, rect2.height);
                        ele1.style.height = maxHeight + "px";
                        ele2.style.height = maxHeight + "px";
                        ele1.setAttribute("data-sync-height", String(maxHeight));
                        ele2.setAttribute("data-sync-height", String(maxHeight));
                    }
                    else {
                        ele1.removeAttribute("data-sync-height");
                        ele2.removeAttribute("data-sync-height");
                    }
                };
                cb();
                const observer = new ResizeObserver(cb);
                const target = ele1.getAttribute("data-side") === side ? ele1 : ele2;
                observer.observe(target);
                target.setAttribute("data-observe", "height");
                return () => {
                    observer.disconnect();
                    target?.removeAttribute("data-observe");
                };
            }
        }
    }, [selector, enable, side, id]);
};

const addContentBGName = "--diff-add-content--";
const delContentBGName = "--diff-del-content--";
const addLineNumberBGName = "--diff-add-lineNumber--";
const delLineNumberBGName = "--diff-del-lineNumber--";
const plainContentBGName = "--diff-plain-content--";
const expandContentBGName = "--diff-expand-content--";
const plainLineNumberColorName = "--diff-plain-lineNumber-color--";
const plainLineNumberBGName = "--diff-plain-lineNumber--";
const hunkContentBGName = "--diff-hunk-content--";
const hunkContentColorName = "--diff-hunk-content-color--";
const hunkLineNumberBGName = "--diff-hunk-lineNumber--";
const addContentHighlightBGName = "--diff-add-content-highlight--";
const delContentHighlightBGName = "--diff-del-content-highlight--";
const addWidgetBGName = "--diff-add-widget--";
const addWidgetColorName = "--diff-add-widget-color--";
const emptyBGName = "--diff-empty-content--";
const getContentBG = (isAdded, isDelete, hasDiff) => {
    return isAdded
        ? `var(${addContentBGName})`
        : isDelete
            ? `var(${delContentBGName})`
            : hasDiff
                ? `var(${plainContentBGName})`
                : `var(${expandContentBGName})`;
};
const getLineNumberBG = (isAdded, isDelete, hasDiff) => {
    return isAdded
        ? `var(${addLineNumberBGName})`
        : isDelete
            ? `var(${delLineNumberBGName})`
            : hasDiff
                ? `var(${plainLineNumberBGName})`
                : `var(${expandContentBGName})`;
};

const _DiffSplitExtendLine$1 = ({ index, diffFile, side, lineNumber, }) => {
    const { useDiffContext } = useDiffViewContext();
    const { extendData, renderExtendLine } = useDiffContext(React.useCallback((s) => ({ extendData: s.extendData, renderExtendLine: s.renderExtendLine }), []));
    const oldLine = diffFile.getSplitLeftLine(index);
    const newLine = diffFile.getSplitRightLine(index);
    const oldLineExtend = extendData?.oldFile?.[oldLine?.lineNumber];
    const newLineExtend = extendData?.newFile?.[newLine?.lineNumber];
    const currentExtend = side === SplitSide.old ? oldLineExtend : newLineExtend;
    const currentLineNumber = side === SplitSide.old ? oldLine.lineNumber : newLine.lineNumber;
    const otherSide = side === SplitSide.old ? SplitSide.new : SplitSide.old;
    useSyncHeight({
        selector: `tr[data-line="${lineNumber}-extend"]`,
        side: currentExtend ? SplitSide[side] : SplitSide[otherSide],
        enable: side === SplitSide.new && typeof renderExtendLine === "function",
    });
    const width = useDomWidth({
        selector: side === SplitSide.old ? ".old-diff-table-wrapper" : ".new-diff-table-wrapper",
        enable: !!currentExtend && typeof renderExtendLine === "function",
    });
    if (!renderExtendLine)
        return null;
    return (React.createElement("tr", { "data-line": `${lineNumber}-extend`, "data-state": "extend", "data-side": SplitSide[side], className: "diff-line diff-line-extend" }, currentExtend ? (React.createElement("td", { className: `diff-line-extend-${SplitSide[side]}-content p-0`, colSpan: 2 },
        React.createElement("div", { className: "diff-line-extend-wrapper sticky left-0", style: { width } }, width > 0 &&
            renderExtendLine?.({
                diffFile,
                side,
                lineNumber: currentLineNumber,
                data: currentExtend.data,
                onUpdate: diffFile.notifyAll,
            })))) : (React.createElement("td", { className: `diff-line-extend-${SplitSide[side]}-placeholder p-0 select-none`, style: { backgroundColor: `var(${emptyBGName})` }, colSpan: 2 },
        React.createElement("span", null, "\u2002")))));
};
const DiffSplitExtendLine$1 = ({ index, diffFile, side, lineNumber, }) => {
    const { useDiffContext } = useDiffViewContext();
    const oldLine = diffFile.getSplitLeftLine(index);
    const newLine = diffFile.getSplitRightLine(index);
    const hasExtend = useDiffContext(React.useCallback((s) => s.extendData?.oldFile?.[oldLine?.lineNumber] || s.extendData?.newFile?.[newLine?.lineNumber], [oldLine?.lineNumber, newLine?.lineNumber]));
    // if the expand action not enabled, the `isHidden` property will never change
    const enableExpand = diffFile.getExpandEnabled();
    const currentLine = side === SplitSide.old ? oldLine : newLine;
    const currentIsShow = hasExtend && (!currentLine.isHidden || !enableExpand);
    if (!currentIsShow)
        return null;
    return React.createElement(_DiffSplitExtendLine$1, { index: index, diffFile: diffFile, side: side, lineNumber: lineNumber });
};

const ExpandDown = ({ className }) => {
    return (React.createElement("svg", { "aria-hidden": "true", height: "16", viewBox: "0 0 16 16", version: "1.1", width: "16", className: className },
        React.createElement("path", { d: "m8.177 14.323 2.896-2.896a.25.25 0 0 0-.177-.427H8.75V7.764a.75.75 0 1 0-1.5 0V11H5.104a.25.25 0 0 0-.177.427l2.896 2.896a.25.25 0 0 0 .354 0ZM2.25 5a.75.75 0 0 0 0-1.5h-.5a.75.75 0 0 0 0 1.5h.5ZM6 4.25a.75.75 0 0 1-.75.75h-.5a.75.75 0 0 1 0-1.5h.5a.75.75 0 0 1 .75.75ZM8.25 5a.75.75 0 0 0 0-1.5h-.5a.75.75 0 0 0 0 1.5h.5ZM12 4.25a.75.75 0 0 1-.75.75h-.5a.75.75 0 0 1 0-1.5h.5a.75.75 0 0 1 .75.75Zm2.25.75a.75.75 0 0 0 0-1.5h-.5a.75.75 0 0 0 0 1.5h.5Z" })));
};
const ExpandUp = ({ className }) => {
    return (React.createElement("svg", { "aria-hidden": "true", height: "16", viewBox: "0 0 16 16", version: "1.1", width: "16", className: className },
        React.createElement("path", { d: "M7.823 1.677 4.927 4.573A.25.25 0 0 0 5.104 5H7.25v3.236a.75.75 0 1 0 1.5 0V5h2.146a.25.25 0 0 0 .177-.427L8.177 1.677a.25.25 0 0 0-.354 0ZM13.75 11a.75.75 0 0 0 0 1.5h.5a.75.75 0 0 0 0-1.5h-.5Zm-3.75.75a.75.75 0 0 1 .75-.75h.5a.75.75 0 0 1 0 1.5h-.5a.75.75 0 0 1-.75-.75ZM7.75 11a.75.75 0 0 0 0 1.5h.5a.75.75 0 0 0 0-1.5h-.5ZM4 11.75a.75.75 0 0 1 .75-.75h.5a.75.75 0 0 1 0 1.5h-.5a.75.75 0 0 1-.75-.75ZM1.75 11a.75.75 0 0 0 0 1.5h.5a.75.75 0 0 0 0-1.5h-.5Z" })));
};
const ExpandAll = ({ className }) => {
    return (React.createElement("svg", { "aria-hidden": "true", height: "16", viewBox: "0 0 16 16", version: "1.1", width: "16", className: className },
        React.createElement("path", { d: "m8.177.677 2.896 2.896a.25.25 0 0 1-.177.427H8.75v1.25a.75.75 0 0 1-1.5 0V4H5.104a.25.25 0 0 1-.177-.427L7.823.677a.25.25 0 0 1 .354 0ZM7.25 10.75a.75.75 0 0 1 1.5 0V12h2.146a.25.25 0 0 1 .177.427l-2.896 2.896a.25.25 0 0 1-.354 0l-2.896-2.896A.25.25 0 0 1 5.104 12H7.25v-1.25Zm-5-2a.75.75 0 0 0 0-1.5h-.5a.75.75 0 0 0 0 1.5h.5ZM6 8a.75.75 0 0 1-.75.75h-.5a.75.75 0 0 1 0-1.5h.5A.75.75 0 0 1 6 8Zm2.25.75a.75.75 0 0 0 0-1.5h-.5a.75.75 0 0 0 0 1.5h.5ZM12 8a.75.75 0 0 1-.75.75h-.5a.75.75 0 0 1 0-1.5h.5A.75.75 0 0 1 12 8Zm2.25.75a.75.75 0 0 0 0-1.5h-.5a.75.75 0 0 0 0 1.5h.5Z" })));
};

const _DiffSplitHunkLine = ({ index, diffFile, side, lineNumber, }) => {
    const currentHunk = diffFile.getSplitHunkLine(index);
    const expandEnabled = diffFile.getExpandEnabled();
    useSyncHeight({
        selector: `tr[data-line="${lineNumber}-hunk"]`,
        side: SplitSide[SplitSide.old],
        enable: side === SplitSide.new,
    });
    const enableHunkAction = side === SplitSide.old;
    const isExpandAll = currentHunk &&
        currentHunk.splitInfo &&
        currentHunk.splitInfo.endHiddenIndex - currentHunk.splitInfo.startHiddenIndex < composeLen;
    const isFirstLine = currentHunk && currentHunk.index === 0;
    const isLastLine = currentHunk && currentHunk.isLast;
    return (React.createElement("tr", { "data-line": `${lineNumber}-hunk`, "data-state": "hunk", "data-side": SplitSide[side], className: "diff-line diff-line-hunk" }, enableHunkAction ? (React.createElement(React.Fragment, null,
        React.createElement("td", { className: "diff-line-hunk-action sticky left-0 p-[1px] w-[1%] min-w-[40px] select-none", style: {
                backgroundColor: `var(${hunkLineNumberBGName})`,
                color: `var(${plainLineNumberColorName})`,
            } }, expandEnabled ? (isFirstLine ? (React.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[6px] cursor-pointer rounded-[2px]", title: "Expand Up", "data-title": "Expand Up", onClick: () => diffFile.onSplitHunkExpand("up", index) },
            React.createElement(ExpandUp, { className: "fill-current" }))) : isLastLine ? (React.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[6px] cursor-pointer rounded-[2px] relative", title: "Expand Down", "data-title": "Expand Down", onClick: () => {
                diffFile.onSplitHunkExpand("down", index);
            } },
            React.createElement(ExpandDown, { className: "fill-current" }))) : isExpandAll ? (React.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[6px] cursor-pointer rounded-[2px]", title: "Expand All", "data-title": "Expand All", onClick: () => diffFile.onSplitHunkExpand("all", index) },
            React.createElement(ExpandAll, { className: "fill-current" }))) : (React.createElement(React.Fragment, null,
            React.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[2px] cursor-pointer rounded-[2px]", title: "Expand Down", "data-title": "Expand Down", onClick: () => diffFile.onSplitHunkExpand("down", index) },
                React.createElement(ExpandDown, { className: "fill-current" })),
            React.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[2px] cursor-pointer rounded-[2px]", title: "Expand Up", "data-title": "Expand Up", onClick: () => diffFile.onSplitHunkExpand("up", index) },
                React.createElement(ExpandUp, { className: "fill-current" }))))) : (React.createElement("div", { className: "min-h-[28px]" }, "\u2002"))),
        React.createElement("td", { className: "diff-line-hunk-content pr-[10px] align-middle", style: { backgroundColor: `var(${hunkContentBGName})` } },
            React.createElement("div", { className: "pl-[1.5em]", style: {
                    color: `var(${hunkContentColorName})`,
                } }, currentHunk.splitInfo.plainText)))) : (React.createElement("td", { className: "diff-line-hunk-placeholder select-none", colSpan: 2, style: { backgroundColor: `var(${hunkContentBGName})` } },
        React.createElement("div", { className: "min-h-[28px]" }, "\u2002")))));
};
const DiffSplitHunkLine$1 = ({ index, diffFile, side, lineNumber, }) => {
    const currentHunk = diffFile.getSplitHunkLine(index);
    const currentIsShow = currentHunk &&
        currentHunk.splitInfo &&
        currentHunk.splitInfo.startHiddenIndex < currentHunk.splitInfo.endHiddenIndex;
    if (!currentIsShow)
        return null;
    return React.createElement(_DiffSplitHunkLine, { index: index, diffFile: diffFile, side: side, lineNumber: lineNumber });
};

const DiffSplitAddWidget = ({ side, className, lineNumber, onWidgetClick, onOpenAddWidget, }) => {
    return (React.createElement("div", { className: "diff-add-widget-wrapper" + (className ? " " + className : ""), style: {
            width: "calc(var(--diff-font-size--) * 1.4)",
            height: "calc(var(--diff-font-size--) * 1.4)",
        } },
        React.createElement("button", { className: "diff-add-widget w-0 h-0 overflow-hidden cursor-pointer rounded-md flex items-center justify-center transition-transform origin-center group-hover:w-full group-hover:h-full hover:scale-110", style: {
                color: `var(${addWidgetColorName})`,
                zIndex: 1,
                fontSize: `1.2em`,
                backgroundColor: `var(${addWidgetBGName})`,
            }, onClick: () => {
                onOpenAddWidget(lineNumber, side);
                onWidgetClick?.(lineNumber, side);
            } }, "+")));
};
const DiffUnifiedAddWidget = ({ lineNumber, side, onWidgetClick, onOpenAddWidget, }) => {
    return (React.createElement("div", { className: "diff-add-widget-wrapper absolute left-[100%] top-[1px] translate-x-[-50%]", style: {
            width: "calc(var(--diff-font-size--) * 1.4)",
            height: "calc(var(--diff-font-size--) * 1.4)",
        } },
        React.createElement("button", { className: "diff-add-widget overflow-hidden cursor-pointer rounded-md w-0 h-0 flex items-center justify-center transition-transform origin-center group-hover:w-full group-hover:h-full hover:scale-110", style: {
                color: `var(${addWidgetColorName})`,
                zIndex: 1,
                fontSize: `1.2em`,
                backgroundColor: `var(${addWidgetBGName})`,
            }, onClick: () => {
                onOpenAddWidget(lineNumber, side);
                onWidgetClick?.(lineNumber, side);
            } }, "+")));
};

const DiffString = ({ rawLine, diffLine, operator, }) => {
    const range = diffLine?.range;
    if (range) {
        const str1 = rawLine.slice(0, range.location);
        const str2 = rawLine.slice(range.location, range.location + range.length);
        const str3 = rawLine.slice(range.location + range.length);
        const isNewLineSymbolChanged = range.isNewLineSymbolChanged;
        return (React.createElement("span", { className: "diff-line-content-raw" },
            React.createElement("span", { "data-range-start": range.location, "data-range-end": range.location + range.length },
                str1,
                React.createElement("span", { "data-diff-highlight": true, className: "rounded-[0.2em]", style: {
                        backgroundColor: operator === "add" ? `var(${addContentHighlightBGName})` : `var(${delContentHighlightBGName})`,
                    } }, isNewLineSymbolChanged
                    ? str2 === "\r"
                        ? "␍"
                        : str2 === "\n"
                            ? "␊"
                            : str2 === "\r\n"
                                ? "␍␊"
                                : str2
                    : str2),
                str3)));
    }
    return React.createElement("span", { className: "diff-line-content-raw" }, rawLine);
};
const DiffSyntax = ({ rawLine, diffLine, operator, syntaxLine, }) => {
    if (!syntaxLine) {
        return React.createElement(DiffString, { rawLine: rawLine, diffLine: diffLine, operator: operator });
    }
    const range = diffLine?.range;
    if (range) {
        return (React.createElement("span", { className: "diff-line-syntax-raw" },
            React.createElement("span", { "data-range-start": range.location, "data-range-end": range.location + range.length }, syntaxLine.nodeList?.map(({ node, wrapper }, index) => {
                if (node.endIndex < range.location || range.location + range.length < node.startIndex) {
                    return (React.createElement("span", { key: index, "data-start": node.startIndex, "data-end": node.endIndex, className: wrapper?.properties?.className?.join(" ") }, node.value));
                }
                else {
                    const index1 = range.location - node.startIndex;
                    const index2 = index1 < 0 ? 0 : index1;
                    const str1 = node.value.slice(0, index2);
                    const str2 = node.value.slice(index2, index1 + range.length);
                    const str3 = node.value.slice(index1 + range.length);
                    const isStart = str1.length || range.location === node.startIndex;
                    const isEnd = str3.length || node.endIndex === range.location + range.length - 1;
                    const isNewLineSymbolChanged = range.isNewLineSymbolChanged;
                    return (React.createElement("span", { key: index, "data-start": node.startIndex, "data-end": node.endIndex, className: wrapper?.properties?.className?.join(" ") },
                        str1,
                        React.createElement("span", { "data-diff-highlight": true, style: {
                                backgroundColor: operator === "add" ? `var(${addContentHighlightBGName})` : `var(${delContentHighlightBGName})`,
                                borderTopLeftRadius: isStart ? "0.2em" : undefined,
                                borderBottomLeftRadius: isStart ? "0.2em" : undefined,
                                borderTopRightRadius: isEnd ? "0.2em" : undefined,
                                borderBottomRightRadius: isEnd ? "0.2em" : undefined,
                            } }, isNewLineSymbolChanged
                            ? str2 === "\r"
                                ? "␍"
                                : str2 === "\n"
                                    ? "␊"
                                    : str2 === "\r\n"
                                        ? "␍␊"
                                        : str2
                            : str2),
                        str3));
                }
            }))));
    }
    return (React.createElement("span", { className: "diff-line-syntax-raw" }, syntaxLine?.nodeList?.map(({ node, wrapper }, index) => (React.createElement("span", { key: index, "data-start": node.startIndex, "data-end": node.endIndex, className: wrapper?.properties?.className?.join(" ") }, node.value)))));
};
const DiffContent = ({ diffLine, rawLine, syntaxLine, enableWrap, enableHighlight, }) => {
    const isAdded = diffLine?.type === DiffLineType.Add;
    const isDelete = diffLine?.type === DiffLineType.Delete;
    const isMaxLineLengthToIgnoreSyntax = syntaxLine?.nodeList?.length > 150;
    return (React.createElement("div", { className: "diff-line-content-item pl-[2.0em]", "data-val": rawLine, style: {
            whiteSpace: enableWrap ? "pre-wrap" : "pre",
            wordBreak: enableWrap ? "break-all" : "initial",
        } },
        React.createElement("span", { "data-operator": isAdded ? "+" : isDelete ? "-" : undefined, className: "diff-line-content-operator inline-block w-[1.5em] ml-[-1.5em] indent-[0.2em] select-none" }, isAdded ? "+" : isDelete ? "-" : " "),
        enableHighlight && syntaxLine && !isMaxLineLengthToIgnoreSyntax ? (React.createElement(DiffSyntax, { operator: isAdded ? "add" : isDelete ? "del" : undefined, rawLine: rawLine, diffLine: diffLine, syntaxLine: syntaxLine })) : (React.createElement(DiffString, { operator: isAdded ? "add" : isDelete ? "del" : undefined, rawLine: rawLine, diffLine: diffLine }))));
};

const DiffWidgetContext = createContext(null);
DiffWidgetContext.displayName = "DiffWidgetContext";
const useDiffWidgetContext = () => useContext(DiffWidgetContext);

const _DiffSplitLine$1 = ({ index, diffFile, lineNumber, side, }) => {
    const getCurrentSyntaxLine = side === SplitSide.old ? diffFile.getOldSyntaxLine : diffFile.getNewSyntaxLine;
    const oldLine = diffFile.getSplitLeftLine(index);
    const newLine = diffFile.getSplitRightLine(index);
    const currentLine = side === SplitSide.old ? oldLine : newLine;
    const hasDiff = !!currentLine?.diff;
    const hasContent = !!currentLine.lineNumber;
    const hasChange = checkDiffLineIncludeChange(currentLine?.diff);
    const isAdded = currentLine?.diff?.type === DiffLineType.Add;
    const isDelete = currentLine?.diff?.type === DiffLineType.Delete;
    const { useDiffContext } = useDiffViewContext();
    const { enableHighlight, enableAddWidget, onAddWidgetClick } = useDiffContext(React.useCallback((s) => ({
        enableHighlight: s.enableHighlight,
        enableAddWidget: s.enableAddWidget,
        onAddWidgetClick: s.onAddWidgetClick,
    }), []));
    const { useWidget } = useDiffWidgetContext();
    const { setWidget } = useWidget.getReadonlyState();
    const contentBG = getContentBG(isAdded, isDelete, hasDiff);
    const lineNumberBG = getLineNumberBG(isAdded, isDelete, hasDiff);
    const syntaxLine = getCurrentSyntaxLine(currentLine.lineNumber);
    return (React.createElement("tr", { "data-line": lineNumber, "data-state": hasDiff || !hasContent ? "diff" : "plain", "data-side": SplitSide[side], className: "diff-line" + (hasContent ? " group" : "") }, hasContent ? (React.createElement(React.Fragment, null,
        React.createElement("td", { className: `diff-line-${SplitSide[side]}-num sticky left-0 pl-[10px] pr-[10px] text-right align-top select-none w-[1%] min-w-[40px]`, style: {
                backgroundColor: lineNumberBG,
                color: `var(${plainLineNumberColorName})`,
            } },
            hasDiff && enableAddWidget && (React.createElement(DiffSplitAddWidget, { index: index, lineNumber: currentLine.lineNumber, side: side, diffFile: diffFile, onWidgetClick: onAddWidgetClick, className: "absolute left-[100%] translate-x-[-50%] z-[1]", onOpenAddWidget: (lineNumber, side) => setWidget({ lineNumber: lineNumber, side: side }) })),
            React.createElement("span", { "data-line-num": currentLine.lineNumber, style: { opacity: hasChange ? undefined : 0.5 } }, currentLine.lineNumber)),
        React.createElement("td", { className: `diff-line-${SplitSide[side]}-content pr-[10px] align-top`, style: { backgroundColor: contentBG } },
            React.createElement(DiffContent, { enableWrap: false, diffFile: diffFile, rawLine: currentLine.value, diffLine: currentLine.diff, syntaxLine: syntaxLine, enableHighlight: enableHighlight })))) : (React.createElement("td", { className: `diff-line-${SplitSide[side]}-placeholder select-none`, style: { backgroundColor: `var(${emptyBGName})` }, colSpan: 2 },
        React.createElement("span", null, "\u2002")))));
};
const DiffSplitLine$1 = ({ index, diffFile, lineNumber, side, }) => {
    const getCurrentLine = side === SplitSide.old ? diffFile.getSplitLeftLine : diffFile.getSplitRightLine;
    const currentLine = getCurrentLine(index);
    if (currentLine?.isHidden)
        return null;
    return React.createElement(_DiffSplitLine$1, { index: index, diffFile: diffFile, lineNumber: lineNumber, side: side });
};

const _DiffSplitWidgetLine$1 = ({ diffFile, side, lineNumber, currentLine, setWidget, currentWidget, }) => {
    const otherSide = side === SplitSide.old ? SplitSide.new : SplitSide.old;
    const { useDiffContext } = useDiffViewContext();
    const renderWidgetLine = useDiffContext(React.useCallback((s) => s.renderWidgetLine, []));
    useSyncHeight({
        selector: `tr[data-line="${lineNumber}-widget"]`,
        side: currentWidget ? SplitSide[side] : SplitSide[otherSide],
        enable: side === SplitSide.new && typeof renderWidgetLine === "function",
    });
    const width = useDomWidth({
        selector: side === SplitSide.old ? ".old-diff-table-wrapper" : ".new-diff-table-wrapper",
        enable: !!currentWidget && typeof renderWidgetLine === "function",
    });
    if (!renderWidgetLine)
        return null;
    return (React.createElement("tr", { "data-line": `${lineNumber}-widget`, "data-state": "widget", "data-side": SplitSide[side], className: "diff-line diff-line-widget" }, currentWidget ? (React.createElement("td", { className: `diff-line-widget-${SplitSide[side]}-content p-0`, colSpan: 2 },
        React.createElement("div", { className: "diff-line-widget-wrapper sticky left-0", style: { width } }, width > 0 &&
            renderWidgetLine?.({
                diffFile,
                side,
                lineNumber: currentLine.lineNumber,
                onClose: () => setWidget({}),
            })))) : (React.createElement("td", { className: `diff-line-widget-${SplitSide[side]}-placeholder p-0`, style: { backgroundColor: `var(${emptyBGName})` }, colSpan: 2 },
        React.createElement("span", null, "\u2002")))));
};
const DiffSplitWidgetLine$1 = ({ index, diffFile, side, lineNumber, }) => {
    const { useWidget } = useDiffWidgetContext();
    const { widgetLineNumber, widgetSide, setWidget } = useWidget(React.useCallback((s) => ({ widgetLineNumber: s.widgetLineNumber, widgetSide: s.widgetSide, setWidget: s.setWidget }), []));
    const oldLine = diffFile.getSplitLeftLine(index);
    const newLine = diffFile.getSplitRightLine(index);
    const oldLineWidget = oldLine.lineNumber && widgetSide === SplitSide.old && widgetLineNumber === oldLine.lineNumber;
    const newLineWidget = newLine.lineNumber && widgetSide === SplitSide.new && widgetLineNumber === newLine.lineNumber;
    const currentLine = side === SplitSide.old ? oldLine : newLine;
    const currentWidget = side === SplitSide.old ? oldLineWidget : newLineWidget;
    const currentIsShow = oldLineWidget || newLineWidget;
    if (!currentIsShow)
        return null;
    return (React.createElement(_DiffSplitWidgetLine$1, { index: index, diffFile: diffFile, side: side, lineNumber: lineNumber, currentLine: currentLine, setWidget: setWidget, currentWidget: currentWidget }));
};

const removeAllSelection = () => {
    const selection = window.getSelection();
    for (let i = 0; i < selection.rangeCount; i++) {
        selection.removeRange(selection.getRangeAt(i));
    }
};
const syncScroll = (left, right) => {
    const onScroll = function (event) {
        if (event === null || event.target === null)
            return;
        if (event.target === left) {
            right.scrollTop = left.scrollTop;
            right.scrollLeft = left.scrollLeft;
        }
        else {
            left.scrollTop = right.scrollTop;
            left.scrollLeft = right.scrollLeft;
        }
    };
    if (!left.onscroll) {
        left.onscroll = onScroll;
    }
    if (!right.onscroll) {
        right.onscroll = onScroll;
    }
    return () => {
        left.onscroll = null;
        right.onscroll = null;
    };
};

const onMouseDown$1 = (e) => {
    const ele = e.target;
    // need remove all the selection
    if (ele && ele instanceof HTMLElement && ele.nodeName === "BUTTON") {
        removeAllSelection();
        return;
    }
};
const DiffSplitViewTable = ({ side, diffFile, width }) => {
    const className = side === SplitSide.new ? "new-diff-table" : "old-diff-table";
    const lines = getSplitContentLines(diffFile);
    return (React.createElement("table", { className: className + " border-collapse w-full", "data-mode": SplitSide[side] },
        React.createElement("colgroup", null,
            React.createElement("col", { className: `diff-table-${SplitSide[side]}-num-col`, style: { minWidth: Math.round(width) + 25 } }),
            React.createElement("col", { className: `diff-table-${SplitSide[side]}-content-col` })),
        React.createElement("thead", { className: "hidden" },
            React.createElement("tr", null,
                React.createElement("th", { scope: "col" },
                    SplitSide[side],
                    " line number"),
                React.createElement("th", { scope: "col" },
                    SplitSide[side],
                    " line content"))),
        React.createElement("tbody", { className: "diff-table-body leading-[1.4]", onMouseDownCapture: onMouseDown$1 },
            lines.map((line) => (React.createElement(Fragment, { key: line.index },
                React.createElement(DiffSplitHunkLine$1, { index: line.index, side: side, lineNumber: line.lineNumber, diffFile: diffFile }),
                React.createElement(DiffSplitLine$1, { index: line.index, side: side, lineNumber: line.lineNumber, diffFile: diffFile }),
                React.createElement(DiffSplitWidgetLine$1, { index: line.index, side: side, lineNumber: line.lineNumber, diffFile: diffFile }),
                React.createElement(DiffSplitExtendLine$1, { index: line.index, side: side, lineNumber: line.lineNumber, diffFile: diffFile })))),
            React.createElement(DiffSplitHunkLine$1, { side: side, index: diffFile.splitLineLength, lineNumber: diffFile.splitLineLength, diffFile: diffFile }))));
};
const DiffSplitViewNormal = memo(({ diffFile }) => {
    const ref1 = useRef(null);
    const ref2 = useRef(null);
    const splitLineLength = diffFile.splitLineLength;
    const { useDiffContext } = useDiffViewContext();
    const fontSize = useDiffContext(useCallback((s) => s.fontSize, []));
    useSyncExternalStore(diffFile.subscribe, diffFile.getUpdateCount);
    useEffect(() => {
        const left = ref1.current;
        const right = ref2.current;
        if (!left || !right)
            return;
        return syncScroll(left, right);
    }, []);
    const width = useTextWidth({
        text: splitLineLength.toString(),
        font: { fontSize: fontSize + "px", fontFamily: "Menlo, Consolas, monospace" },
    });
    return (React.createElement("div", { className: "split-diff-view split-diff-view-wrap w-full flex basis-[50%]" },
        React.createElement("div", { className: "old-diff-table-wrapper overflow-auto w-full scrollbar-hide scrollbar-disable", ref: ref1, style: {
                overscrollBehaviorX: "none",
                fontFamily: "Menlo, Consolas, monospace",
                fontSize: "var(--diff-font-size--)",
            } },
            React.createElement(DiffSplitViewTable, { side: SplitSide.old, diffFile: diffFile, width: width })),
        React.createElement("div", { className: "diff-split-line w-[1.5px] bg-[#ccc]" }),
        React.createElement("div", { className: "new-diff-table-wrapper overflow-auto w-full scrollbar-hide scrollbar-disable", ref: ref2, style: {
                overscrollBehaviorX: "none",
                fontFamily: "Menlo, Consolas, monospace",
                fontSize: "var(--diff-font-size--)",
            } },
            React.createElement(DiffSplitViewTable, { side: SplitSide.new, diffFile: diffFile, width: width }))));
});
DiffSplitViewNormal.displayName = "DiffSplitViewNormal";

const _DiffSplitExtendLine = ({ index, diffFile, lineNumber, }) => {
    const { useDiffContext } = useDiffViewContext();
    // 需要显示的时候才进行方法订阅，可以大幅度提高性能
    const { extendData, renderExtendLine } = useDiffContext(React.useCallback((s) => ({ extendData: s.extendData, renderExtendLine: s.renderExtendLine }), []));
    const oldLine = diffFile.getSplitLeftLine(index);
    const newLine = diffFile.getSplitRightLine(index);
    const oldLineExtend = extendData?.oldFile?.[oldLine?.lineNumber];
    const newLineExtend = extendData?.newFile?.[newLine?.lineNumber];
    if (!renderExtendLine)
        return null;
    return (React.createElement("tr", { "data-line": `${lineNumber}-extend`, "data-state": "extend", className: "diff-line diff-line-extend" },
        oldLineExtend ? (React.createElement("td", { className: "diff-line-extend-old-content p-0", colSpan: 2 },
            React.createElement("div", { className: "diff-line-extend-wrapper" }, renderExtendLine?.({
                diffFile,
                side: SplitSide.old,
                lineNumber: oldLine.lineNumber,
                data: oldLineExtend.data,
                onUpdate: diffFile.notifyAll,
            })))) : (React.createElement("td", { className: "diff-line-extend-old-placeholder p-0 select-none", style: { backgroundColor: `var(${emptyBGName})` }, colSpan: 2 },
            React.createElement("span", null, "\u2002"))),
        newLineExtend ? (React.createElement("td", { className: "diff-line-extend-new-content p-0 border-l-[1px] border-l-[#ccc]", colSpan: 2 },
            React.createElement("div", { className: "diff-line-extend-wrapper" }, renderExtendLine?.({
                diffFile,
                side: SplitSide.new,
                lineNumber: newLine.lineNumber,
                data: newLineExtend.data,
                onUpdate: diffFile.notifyAll,
            })))) : (React.createElement("td", { className: "diff-line-extend-new-placeholder p-0 border-l-[1px] border-l-[#ccc] select-none", style: { backgroundColor: `var(${emptyBGName})` }, colSpan: 2 },
            React.createElement("span", null, "\u2002")))));
};
const DiffSplitExtendLine = ({ index, diffFile, lineNumber, }) => {
    const { useDiffContext } = useDiffViewContext();
    const oldLine = diffFile.getSplitLeftLine(index);
    const newLine = diffFile.getSplitRightLine(index);
    const hasExtend = useDiffContext(React.useCallback((s) => s.extendData?.oldFile?.[oldLine?.lineNumber] || s.extendData?.newFile?.[newLine?.lineNumber], [oldLine?.lineNumber, newLine?.lineNumber]));
    // if the expand action not enabled, the `isHidden` property will never change
    const enableExpand = diffFile.getExpandEnabled();
    const currentIsShow = hasExtend && ((!oldLine?.isHidden && !newLine?.isHidden) || !enableExpand);
    if (!currentIsShow)
        return null;
    return React.createElement(_DiffSplitExtendLine, { index: index, diffFile: diffFile, lineNumber: lineNumber });
};

const DiffSplitHunkLine = ({ index, diffFile, lineNumber, }) => {
    const currentHunk = diffFile.getSplitHunkLine(index);
    const expandEnabled = diffFile.getExpandEnabled();
    const isExpandAll = currentHunk &&
        currentHunk.splitInfo &&
        currentHunk.splitInfo.endHiddenIndex - currentHunk.splitInfo.startHiddenIndex < composeLen;
    const currentIsShow = currentHunk &&
        currentHunk.splitInfo &&
        currentHunk.splitInfo.startHiddenIndex < currentHunk.splitInfo.endHiddenIndex;
    const isFirstLine = currentHunk && currentHunk.index === 0;
    const isLastLine = currentHunk && currentHunk.isLast;
    if (!currentIsShow)
        return null;
    return (React.createElement("tr", { "data-line": `${lineNumber}-hunk`, "data-state": "hunk", className: "diff-line diff-line-hunk" },
        React.createElement("td", { className: "diff-line-hunk-action p-[1px] w-[1%] min-w-[40px] select-none", style: {
                backgroundColor: `var(${hunkLineNumberBGName})`,
                color: `var(${plainLineNumberColorName})`,
            } }, expandEnabled ? (isFirstLine ? (React.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[6px] cursor-pointer rounded-[2px]", title: "Expand Up", "data-title": "Expand Up", onClick: () => diffFile.onSplitHunkExpand("up", index) },
            React.createElement(ExpandUp, { className: "fill-current" }))) : isLastLine ? (React.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[6px] cursor-pointer rounded-[2px]", title: "Expand Down", "data-title": "Expand Down", onClick: () => diffFile.onSplitHunkExpand("down", index) },
            React.createElement(ExpandDown, { className: "fill-current" }))) : isExpandAll ? (React.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[6px] cursor-pointer rounded-[2px]", title: "Expand All", "data-title": "Expand All", onClick: () => diffFile.onSplitHunkExpand("all", index) },
            React.createElement(ExpandAll, { className: "fill-current" }))) : (React.createElement(React.Fragment, null,
            React.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[2px] cursor-pointer rounded-[2px]", title: "Expand Down", "data-title": "Expand Down", onClick: () => diffFile.onSplitHunkExpand("down", index) },
                React.createElement(ExpandDown, { className: "fill-current" })),
            React.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[2px] cursor-pointer rounded-[2px]", title: "Expand Up", "data-title": "Expand Up", onClick: () => diffFile.onSplitHunkExpand("up", index) },
                React.createElement(ExpandUp, { className: "fill-current" }))))) : (React.createElement("div", { className: "min-h-[28px]" }, "\u2002"))),
        React.createElement("td", { className: "diff-line-hunk-content pr-[10px] align-middle", style: { backgroundColor: `var(${hunkContentBGName})` }, colSpan: 3 },
            React.createElement("div", { className: "pl-[1.5em]", style: {
                    color: `var(${hunkContentColorName})`,
                } }, currentHunk.splitInfo.plainText))));
};

const _DiffSplitLine = ({ index, diffFile, lineNumber }) => {
    const oldLine = diffFile.getSplitLeftLine(index);
    const newLine = diffFile.getSplitRightLine(index);
    const oldSyntaxLine = diffFile.getOldSyntaxLine(oldLine?.lineNumber);
    const newSyntaxLine = diffFile.getNewSyntaxLine(newLine?.lineNumber);
    const hasDiff = !!oldLine?.diff || !!newLine?.diff;
    const hasChange = checkDiffLineIncludeChange(oldLine?.diff) || checkDiffLineIncludeChange(newLine?.diff);
    const oldLineIsDelete = oldLine?.diff?.type === DiffLineType.Delete;
    const newLineIsAdded = newLine?.diff?.type === DiffLineType.Add;
    const { useDiffContext } = useDiffViewContext();
    const { enableHighlight, enableAddWidget, onAddWidgetClick } = useDiffContext(React.useCallback((s) => ({
        enableHighlight: s.enableHighlight,
        enableAddWidget: s.enableAddWidget,
        onAddWidgetClick: s.onAddWidgetClick,
    }), []));
    const { useWidget } = useDiffWidgetContext();
    const { setWidget } = useWidget.getReadonlyState();
    const hasOldLine = !!oldLine.lineNumber;
    const hasNewLine = !!newLine.lineNumber;
    const oldLineContentBG = getContentBG(false, oldLineIsDelete, hasDiff);
    const oldLineNumberBG = getLineNumberBG(false, oldLineIsDelete, hasDiff);
    const newLineContentBG = getContentBG(newLineIsAdded, false, hasDiff);
    const newLineNumberBG = getLineNumberBG(newLineIsAdded, false, hasDiff);
    return (React.createElement("tr", { "data-line": lineNumber, "data-state": hasDiff ? "diff" : "plain", className: "diff-line" },
        hasOldLine ? (React.createElement(React.Fragment, null,
            React.createElement("td", { className: "diff-line-old-num group relative pl-[10px] pr-[10px] text-right align-top select-none w-[1%] min-w-[40px]", "data-side": SplitSide[SplitSide.old], style: { backgroundColor: oldLineNumberBG, color: `var(${plainLineNumberColorName})` } },
                hasDiff && enableAddWidget && (React.createElement(DiffSplitAddWidget, { index: index, lineNumber: oldLine.lineNumber, side: SplitSide.old, diffFile: diffFile, onWidgetClick: onAddWidgetClick, className: "absolute left-[100%] translate-x-[-50%] z-[1]", onOpenAddWidget: (lineNumber, side) => setWidget({ lineNumber: lineNumber, side: side }) })),
                React.createElement("span", { "data-line-num": oldLine.lineNumber, style: { opacity: hasChange ? undefined : 0.5 } }, oldLine.lineNumber)),
            React.createElement("td", { className: "diff-line-old-content group relative pr-[10px] align-top", "data-side": SplitSide[SplitSide.old], style: { backgroundColor: oldLineContentBG } },
                hasDiff && enableAddWidget && (React.createElement(DiffSplitAddWidget, { index: index, lineNumber: oldLine.lineNumber, side: SplitSide.old, diffFile: diffFile, onWidgetClick: onAddWidgetClick, className: "absolute right-[100%] translate-x-[50%] z-[1] select-none", onOpenAddWidget: (lineNumber, side) => setWidget({ lineNumber: lineNumber, side: side }) })),
                React.createElement(DiffContent, { enableWrap: true, diffFile: diffFile, rawLine: oldLine.value, diffLine: oldLine.diff, syntaxLine: oldSyntaxLine, enableHighlight: enableHighlight })))) : (React.createElement("td", { className: "diff-line-old-placeholder select-none", "data-side": SplitSide[SplitSide.old], style: { backgroundColor: `var(${emptyBGName})` }, colSpan: 2 },
            React.createElement("span", null, "\u2002"))),
        hasNewLine ? (React.createElement(React.Fragment, null,
            React.createElement("td", { className: "diff-line-new-num group relative pl-[10px] pr-[10px] text-right align-top select-none w-[1%] min-w-[40px] border-l-[1px] border-l-[#ccc]", "data-side": SplitSide[SplitSide.new], style: { backgroundColor: newLineNumberBG, color: `var(${plainLineNumberColorName})` } },
                hasDiff && enableAddWidget && (React.createElement(DiffSplitAddWidget, { index: index, lineNumber: newLine.lineNumber, side: SplitSide.new, diffFile: diffFile, onWidgetClick: onAddWidgetClick, className: "absolute left-[100%] translate-x-[-50%] z-[1]", onOpenAddWidget: (lineNumber, side) => setWidget({ lineNumber: lineNumber, side: side }) })),
                React.createElement("span", { "data-line-num": newLine.lineNumber, style: { opacity: hasChange ? undefined : 0.5 } }, newLine.lineNumber)),
            React.createElement("td", { className: "diff-line-new-content group relative pr-[10px] align-top", "data-side": SplitSide[SplitSide.new], style: { backgroundColor: newLineContentBG } },
                hasDiff && enableAddWidget && (React.createElement(DiffSplitAddWidget, { index: index, lineNumber: newLine.lineNumber, side: SplitSide.new, diffFile: diffFile, onWidgetClick: onAddWidgetClick, className: "absolute right-[100%] translate-x-[50%] z-[1] select-none", onOpenAddWidget: (lineNumber, side) => setWidget({ lineNumber: lineNumber, side: side }) })),
                React.createElement(DiffContent, { enableWrap: true, diffFile: diffFile, rawLine: newLine.value || "", diffLine: newLine.diff, syntaxLine: newSyntaxLine, enableHighlight: enableHighlight })))) : (React.createElement("td", { className: "diff-line-new-placeholder select-none border-l-[1px] border-l-[#ccc]", style: { backgroundColor: `var(${emptyBGName})` }, "data-side": SplitSide[SplitSide.new], colSpan: 2 },
            React.createElement("span", null, "\u2002")))));
};
const DiffSplitLine = ({ index, diffFile, lineNumber, }) => {
    const oldLine = diffFile.getSplitLeftLine(index);
    const newLine = diffFile.getSplitRightLine(index);
    if (oldLine?.isHidden && newLine?.isHidden)
        return null;
    return React.createElement(_DiffSplitLine, { index: index, diffFile: diffFile, lineNumber: lineNumber });
};

const _DiffSplitWidgetLine = ({ index, diffFile, lineNumber, }) => {
    const { useWidget } = useDiffWidgetContext();
    const { widgetLineNumber, widgetSide, setWidget } = useWidget(React.useCallback((s) => ({ widgetLineNumber: s.widgetLineNumber, widgetSide: s.widgetSide, setWidget: s.setWidget }), []));
    const { useDiffContext } = useDiffViewContext();
    const renderWidgetLine = useDiffContext(React.useCallback((s) => s.renderWidgetLine, []));
    const oldLine = diffFile.getSplitLeftLine(index);
    const newLine = diffFile.getSplitRightLine(index);
    const oldLineWidget = oldLine.lineNumber && widgetSide === SplitSide.old && widgetLineNumber === oldLine.lineNumber;
    const newLineWidget = newLine.lineNumber && widgetSide === SplitSide.new && widgetLineNumber === newLine.lineNumber;
    if (!renderWidgetLine)
        return null;
    return (React.createElement("tr", { "data-line": `${lineNumber}-widget`, "data-state": "widget", className: "diff-line diff-line-widget" },
        oldLineWidget ? (React.createElement("td", { className: "diff-line-widget-old-content p-0", colSpan: 2 },
            React.createElement("div", { className: "diff-line-widget-wrapper" }, renderWidgetLine?.({
                diffFile,
                side: SplitSide.old,
                lineNumber: oldLine.lineNumber,
                onClose: () => setWidget({}),
            })))) : (React.createElement("td", { className: "diff-line-widget-old-placeholder p-0 select-none", style: { backgroundColor: `var(${emptyBGName})` }, colSpan: 2 },
            React.createElement("span", null, "\u2002"))),
        newLineWidget ? (React.createElement("td", { className: "diff-line-widget-new-content p-0 border-l-[1px] border-l-[#ccc]", colSpan: 2 },
            React.createElement("div", { className: "diff-line-widget-wrapper" }, renderWidgetLine?.({
                diffFile,
                side: SplitSide.new,
                lineNumber: newLine.lineNumber,
                onClose: () => setWidget({}),
            })))) : (React.createElement("td", { className: "diff-line-widget-new-placeholder p-0 border-l-[1px] border-l-[#ccc] select-none", style: { backgroundColor: `var(${emptyBGName})` }, colSpan: 2 },
            React.createElement("span", null, "\u2002")))));
};
const DiffSplitWidgetLine = ({ index, diffFile, lineNumber, }) => {
    const { useWidget } = useDiffWidgetContext();
    const { widgetLineNumber, widgetSide } = useWidget(React.useCallback((s) => ({ widgetLineNumber: s.widgetLineNumber, widgetSide: s.widgetSide }), []));
    const oldLine = diffFile.getSplitLeftLine(index);
    const newLine = diffFile.getSplitRightLine(index);
    const oldLineWidget = oldLine.lineNumber && widgetSide === SplitSide.old && widgetLineNumber === oldLine.lineNumber;
    const newLineWidget = newLine.lineNumber && widgetSide === SplitSide.new && widgetLineNumber === newLine.lineNumber;
    const currentIsShow = oldLineWidget || newLineWidget;
    if (!currentIsShow)
        return null;
    return React.createElement(_DiffSplitWidgetLine, { index: index, diffFile: diffFile, lineNumber: lineNumber });
};

const Style = ({ useSelector, id, }) => {
    const splitRef = useSelector((s) => s.splitRef);
    return (React.createElement("style", null, splitRef === SplitSide.old
        ? `#${id} td[data-side="${SplitSide[SplitSide.new]}"] {user-select: none}`
        : splitRef === SplitSide.new
            ? `#${id} td[data-side="${SplitSide[SplitSide.old]}"] {user-select: none}`
            : ""));
};
const DiffSplitViewWrap = memo(({ diffFile }) => {
    const splitLineLength = diffFile.splitLineLength;
    const { useDiffContext } = useDiffViewContext();
    const splitSideInfo = useMemo(() => createStore(() => {
        const splitRef = ref(undefined);
        const setSplit = (side) => {
            flushSync(() => {
                splitRef.value = side;
            });
        };
        return { splitRef, setSplit };
    }), []);
    const setSelectSide = splitSideInfo.getReadonlyState().setSplit;
    const fontSize = useDiffContext(useCallback((s) => s.fontSize, []));
    useSyncExternalStore(diffFile.subscribe, diffFile.getUpdateCount);
    const onMouseDown = useCallback((e) => {
        let ele = e.target;
        // need remove all the selection
        if (ele && ele instanceof HTMLElement && ele.nodeName === "BUTTON") {
            removeAllSelection();
            return;
        }
        while (ele && ele instanceof HTMLElement && ele.nodeName !== "TD") {
            ele = ele.parentElement;
        }
        if (ele instanceof HTMLElement) {
            const side = ele.getAttribute("data-side");
            if (side) {
                setSelectSide(SplitSide[side]);
                removeAllSelection();
            }
        }
    }, [setSelectSide]);
    const width = useTextWidth({
        text: splitLineLength.toString(),
        font: { fontSize: fontSize + "px", fontFamily: "Menlo, Consolas, monospace" },
    });
    const lines = getSplitContentLines(diffFile);
    return (React.createElement("div", { className: "split-diff-view split-diff-view-normal w-full" },
        React.createElement("div", { className: "diff-table-wrapper w-full", style: {
                fontFamily: "Menlo, Consolas, monospace",
                fontSize: "var(--diff-font-size--)",
            } },
            React.createElement(Style, { useSelector: splitSideInfo, id: `diff-root${diffFile.getId()}` }),
            React.createElement("table", { className: "diff-table border-collapse table-fixed w-full" },
                React.createElement("colgroup", null,
                    React.createElement("col", { className: "diff-table-old-num-col", width: Math.round(width) + 25 }),
                    React.createElement("col", { className: "diff-table-old-content-col" }),
                    React.createElement("col", { className: "diff-table-new-num-col", width: Math.round(width) + 25 }),
                    React.createElement("col", { className: "diff-table-new-content-col" })),
                React.createElement("thead", { className: "hidden" },
                    React.createElement("tr", null,
                        React.createElement("th", { scope: "col" }, "old line number"),
                        React.createElement("th", { scope: "col" }, "old line content"),
                        React.createElement("th", { scope: "col" }, "new line number"),
                        React.createElement("th", { scope: "col" }, "new line content"))),
                React.createElement("tbody", { className: "diff-table-body leading-[1.4]", onMouseDownCapture: onMouseDown },
                    lines.map((line) => (React.createElement(Fragment, { key: line.index },
                        React.createElement(DiffSplitHunkLine, { index: line.index, lineNumber: line.lineNumber, diffFile: diffFile }),
                        React.createElement(DiffSplitLine, { index: line.index, lineNumber: line.lineNumber, diffFile: diffFile }),
                        React.createElement(DiffSplitWidgetLine, { index: line.index, lineNumber: line.lineNumber, diffFile: diffFile }),
                        React.createElement(DiffSplitExtendLine, { index: line.index, lineNumber: line.lineNumber, diffFile: diffFile })))),
                    React.createElement(DiffSplitHunkLine, { index: diffFile.splitLineLength, lineNumber: diffFile.splitLineLength, diffFile: diffFile }))))));
});
DiffSplitViewWrap.displayName = "DiffSplitViewWrap";

const DiffSplitView = memo(({ diffFile }) => {
    const { useDiffContext } = useDiffViewContext();
    const enableWrap = useDiffContext(React.useCallback((s) => s.enableWrap, []));
    // performance optimization
    const useWidget = useMemo(() => createStore(() => {
        const widgetSide = ref(undefined);
        const widgetLineNumber = ref(undefined);
        const setWidget = ({ side, lineNumber }) => {
            const { renderWidgetLine } = useDiffContext.getReadonlyState();
            if (typeof renderWidgetLine !== "function")
                return;
            widgetSide.value = side;
            widgetLineNumber.value = lineNumber;
        };
        return { widgetSide, widgetLineNumber, setWidget };
    }), [useDiffContext]);
    const contextValue = useMemo(() => ({ useWidget }), [useWidget]);
    useSyncExternalStore(diffFile.subscribe, diffFile.getUpdateCount);
    useEffect(() => {
        const { setWidget } = useWidget.getReadonlyState();
        setWidget({});
    }, [diffFile, useWidget]);
    return (React.createElement(DiffWidgetContext.Provider, { value: contextValue }, enableWrap ? React.createElement(DiffSplitViewWrap, { diffFile: diffFile }) : React.createElement(DiffSplitViewNormal, { diffFile: diffFile })));
});
DiffSplitView.displayName = "DiffSplitView";

const _DiffUnifiedExtendLine = ({ index, diffFile, lineNumber, }) => {
    const { useDiffContext } = useDiffViewContext();
    const renderExtendLine = useDiffContext(useCallback((s) => s.renderExtendLine, []));
    const unifiedItem = diffFile.getUnifiedLine(index);
    const oldExtend = useDiffContext(useCallback((s) => s.extendData?.oldFile?.[unifiedItem?.oldLineNumber], [unifiedItem?.oldLineNumber]));
    const newExtend = useDiffContext(useCallback((s) => s.extendData?.newFile?.[unifiedItem?.newLineNumber], [unifiedItem?.newLineNumber]));
    const width = useDomWidth({
        selector: ".unified-diff-table-wrapper",
        enable: typeof renderExtendLine === "function",
    });
    if (!renderExtendLine)
        return null;
    return (React.createElement("tr", { "data-line": `${lineNumber}-extend`, "data-state": "extend", className: "diff-line diff-line-extend" },
        React.createElement("td", { className: "diff-line-extend-content align-top p-0", colSpan: 2 },
            React.createElement("div", { className: "diff-line-extend-wrapper sticky left-0", style: { width } },
                width > 0 &&
                    oldExtend &&
                    renderExtendLine?.({
                        diffFile,
                        side: SplitSide.old,
                        lineNumber: unifiedItem.oldLineNumber,
                        data: oldExtend.data,
                        onUpdate: diffFile.notifyAll,
                    }),
                width > 0 &&
                    newExtend &&
                    renderExtendLine?.({
                        diffFile,
                        side: SplitSide.new,
                        lineNumber: unifiedItem.newLineNumber,
                        data: newExtend.data,
                        onUpdate: diffFile.notifyAll,
                    })))));
};
const DiffUnifiedExtendLine = ({ index, diffFile, lineNumber, }) => {
    const { useDiffContext } = useDiffViewContext();
    const unifiedItem = diffFile.getUnifiedLine(index);
    const hasExtend = useDiffContext(useCallback((s) => s.extendData?.oldFile?.[unifiedItem?.oldLineNumber] || s.extendData?.newFile?.[unifiedItem?.newLineNumber], [unifiedItem.oldLineNumber, unifiedItem.newLineNumber]));
    if (!hasExtend || !unifiedItem || unifiedItem.isHidden || !unifiedItem.diff)
        return null;
    return React.createElement(_DiffUnifiedExtendLine, { index: index, diffFile: diffFile, lineNumber: lineNumber });
};

const _DiffUnifiedHunkLine = ({ index, diffFile, lineNumber, }) => {
    const currentHunk = diffFile.getUnifiedHunkLine(index);
    const expandEnabled = diffFile.getExpandEnabled();
    const { useDiffContext } = useDiffViewContext();
    const enableWrap = useDiffContext(useCallback((s) => s.enableWrap, []));
    const isExpandAll = currentHunk &&
        currentHunk.unifiedInfo &&
        currentHunk.unifiedInfo.endHiddenIndex - currentHunk.unifiedInfo.startHiddenIndex < composeLen;
    const isFirstLine = currentHunk && currentHunk.index === 0;
    const isLastLine = currentHunk && currentHunk.isLast;
    return (React.createElement("tr", { "data-line": `${lineNumber}-hunk`, "data-state": "hunk", className: "diff-line diff-line-hunk" },
        React.createElement("td", { className: "diff-line-hunk-action sticky left-0 p-[1px] w-[1%] min-w-[100px] select-none", style: {
                backgroundColor: `var(${hunkLineNumberBGName})`,
                color: `var(${plainLineNumberColorName})`,
            } }, expandEnabled ? (isFirstLine ? (React.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[6px] cursor-pointer rounded-[2px]", title: "Expand Up", "data-title": "Expand Up", onClick: () => diffFile.onUnifiedHunkExpand("up", index) },
            React.createElement(ExpandUp, { className: "fill-current" }))) : isLastLine ? (React.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[6px] cursor-pointer rounded-[2px]", title: "Expand Down", "data-title": "Expand Down", onClick: () => diffFile.onUnifiedHunkExpand("down", index) },
            React.createElement(ExpandDown, { className: "fill-current" }))) : isExpandAll ? (React.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[6px] cursor-pointer rounded-[2px]", title: "Expand All", "data-title": "Expand All", onClick: () => diffFile.onUnifiedHunkExpand("all", index) },
            React.createElement(ExpandAll, { className: "fill-current" }))) : (React.createElement(React.Fragment, null,
            React.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[2px] cursor-pointer rounded-[2px]", title: "Expand Down", "data-title": "Expand Down", onClick: () => diffFile.onUnifiedHunkExpand("down", index) },
                React.createElement(ExpandDown, { className: "fill-current" })),
            React.createElement("button", { className: "w-full diff-widget-tooltip hover:bg-blue-300 flex justify-center items-center py-[2px] cursor-pointer rounded-[2px]", title: "Expand Up", "data-title": "Expand Up", onClick: () => diffFile.onUnifiedHunkExpand("up", index) },
                React.createElement(ExpandUp, { className: "fill-current" }))))) : (React.createElement("div", { className: "min-h-[28px]" }, "\u2002"))),
        React.createElement("td", { className: "diff-line-hunk-content pr-[10px] align-middle", style: { backgroundColor: `var(${hunkContentBGName})` } },
            React.createElement("div", { className: "pl-[1.5em]", style: {
                    whiteSpace: enableWrap ? "pre-wrap" : "pre",
                    wordBreak: enableWrap ? "break-all" : "initial",
                    color: `var(${hunkContentColorName})`,
                } }, currentHunk.unifiedInfo.plainText))));
};
const DiffUnifiedHunkLine = ({ index, diffFile, lineNumber, }) => {
    const currentHunk = diffFile.getUnifiedHunkLine(index);
    const currentIsShow = currentHunk &&
        currentHunk.unifiedInfo &&
        currentHunk.unifiedInfo.startHiddenIndex < currentHunk.unifiedInfo.endHiddenIndex;
    if (!currentIsShow)
        return null;
    return React.createElement(_DiffUnifiedHunkLine, { index: index, diffFile: diffFile, lineNumber: lineNumber });
};

const DiffUnifiedOldLine = ({ index, diffLine, rawLine, syntaxLine, lineNumber, diffFile, setWidget, enableWrap, enableAddWidget, enableHighlight, onAddWidgetClick, }) => {
    return (React.createElement("tr", { "data-line": index, "data-state": "diff", className: "diff-line group" },
        React.createElement("td", { className: "diff-line-num sticky left-0 pl-[10px] pr-[10px] text-right select-none w-[1%] min-w-[100px] whitespace-nowrap align-top", style: {
                color: `var(${plainLineNumberColorName})`,
                backgroundColor: `var(${delLineNumberBGName})`,
            } },
            enableAddWidget && (React.createElement(DiffUnifiedAddWidget, { index: index - 1, lineNumber: lineNumber, diffFile: diffFile, side: SplitSide.old, onWidgetClick: onAddWidgetClick, onOpenAddWidget: (lineNumber, side) => setWidget({ lineNumber, side }) })),
            React.createElement("div", { className: "flex" },
                React.createElement("span", { "data-line-old-num": lineNumber, className: "inline-block w-[50%]" }, lineNumber),
                React.createElement("span", { className: "w-[10px] shrink-0" }),
                React.createElement("span", { className: "inline-block w-[50%]" }))),
        React.createElement("td", { className: "diff-line-content pr-[10px] align-top", style: { backgroundColor: `var(${delContentBGName})` } },
            React.createElement(DiffContent, { enableWrap: enableWrap, diffFile: diffFile, enableHighlight: enableHighlight, rawLine: rawLine, diffLine: diffLine, syntaxLine: syntaxLine }))));
};
const DiffUnifiedNewLine = ({ index, diffLine, rawLine, syntaxLine, lineNumber, diffFile, setWidget, enableWrap, enableAddWidget, enableHighlight, onAddWidgetClick, }) => {
    return (React.createElement("tr", { "data-line": index, "data-state": "diff", className: "diff-line group" },
        React.createElement("td", { className: "diff-line-num sticky left-0 pl-[10px] pr-[10px] text-right select-none w-[1%] min-w-[100px] whitespace-nowrap align-top", style: {
                color: `var(${plainLineNumberColorName})`,
                backgroundColor: `var(${addLineNumberBGName})`,
            } },
            enableAddWidget && (React.createElement(DiffUnifiedAddWidget, { index: index - 1, lineNumber: lineNumber, diffFile: diffFile, side: SplitSide.new, onWidgetClick: onAddWidgetClick, onOpenAddWidget: (lineNumber, side) => setWidget({ lineNumber, side }) })),
            React.createElement("div", { className: "flex" },
                React.createElement("span", { className: "inline-block w-[50%]" }),
                React.createElement("span", { className: "shrink-0 w-[10px]" }),
                React.createElement("span", { "data-line-new-num": lineNumber, className: "inline-block w-[50%]" }, lineNumber))),
        React.createElement("td", { className: "diff-line-content pr-[10px] align-top", style: { backgroundColor: `var(${addContentBGName})` } },
            React.createElement(DiffContent, { enableWrap: enableWrap, diffFile: diffFile, enableHighlight: enableHighlight, rawLine: rawLine, diffLine: diffLine, syntaxLine: syntaxLine }))));
};
const _DiffUnifiedLine = memo(({ index, diffFile, lineNumber }) => {
    const unifiedLine = diffFile.getUnifiedLine(index);
    const { useDiffContext } = useDiffViewContext();
    const { enableWrap, enableHighlight, enableAddWidget, onAddWidgetClick } = useDiffContext(React.useCallback((s) => ({
        enableWrap: s.enableWrap,
        enableHighlight: s.enableHighlight,
        enableAddWidget: s.enableAddWidget,
        onAddWidgetClick: s.onAddWidgetClick,
    }), []));
    const { useWidget } = useDiffWidgetContext();
    const { setWidget } = useWidget.getReadonlyState();
    const hasDiff = unifiedLine.diff;
    const hasChange = checkDiffLineIncludeChange(unifiedLine.diff);
    const rawLine = unifiedLine.value || "";
    const diffLine = unifiedLine.diff;
    const newLineNumber = unifiedLine.newLineNumber;
    const oldLinenumber = unifiedLine.oldLineNumber;
    const syntaxLine = newLineNumber
        ? diffFile.getNewSyntaxLine(newLineNumber)
        : oldLinenumber
            ? diffFile.getOldSyntaxLine(oldLinenumber)
            : undefined;
    if (hasChange) {
        if (unifiedLine.oldLineNumber) {
            return (React.createElement(DiffUnifiedOldLine, { index: lineNumber, enableWrap: enableWrap, diffFile: diffFile, rawLine: rawLine, diffLine: diffLine, setWidget: setWidget, syntaxLine: syntaxLine, enableHighlight: enableHighlight, enableAddWidget: enableAddWidget, onAddWidgetClick: onAddWidgetClick, lineNumber: unifiedLine.oldLineNumber }));
        }
        else {
            return (React.createElement(DiffUnifiedNewLine, { index: lineNumber, enableWrap: enableWrap, rawLine: rawLine, diffLine: diffLine, diffFile: diffFile, setWidget: setWidget, syntaxLine: syntaxLine, enableHighlight: enableHighlight, enableAddWidget: enableAddWidget, onAddWidgetClick: onAddWidgetClick, lineNumber: unifiedLine.newLineNumber }));
        }
    }
    else {
        return (React.createElement("tr", { "data-line": lineNumber, "data-state": unifiedLine.diff ? "diff" : "plain", className: "diff-line group" },
            React.createElement("td", { className: "diff-line-num sticky left-0 pl-[10px] pr-[10px] text-right select-none w-[1%] min-w-[100px] whitespace-nowrap align-top", style: {
                    color: `var(${plainLineNumberColorName})`,
                    backgroundColor: hasDiff ? `var(${plainLineNumberBGName})` : `var(${expandContentBGName})`,
                } },
                enableAddWidget && hasDiff && (React.createElement(DiffUnifiedAddWidget, { index: index, diffFile: diffFile, lineNumber: unifiedLine.newLineNumber, side: SplitSide.new, onWidgetClick: onAddWidgetClick, onOpenAddWidget: (lineNumber, side) => setWidget({ lineNumber, side }) })),
                React.createElement("div", { className: "flex opacity-[0.5]" },
                    React.createElement("span", { "data-line-old-num": unifiedLine.oldLineNumber, className: "inline-block w-[50%]" }, unifiedLine.oldLineNumber),
                    React.createElement("span", { className: "w-[10px] shrink-0" }),
                    React.createElement("span", { "data-line-new-num": unifiedLine.newLineNumber, className: "inline-block w-[50%]" }, unifiedLine.newLineNumber))),
            React.createElement("td", { className: "diff-line-content pr-[10px] align-top", style: {
                    backgroundColor: hasDiff ? `var(${plainContentBGName})` : `var(${expandContentBGName})`,
                } },
                React.createElement(DiffContent, { enableWrap: enableWrap, diffFile: diffFile, enableHighlight: enableHighlight, rawLine: rawLine, diffLine: diffLine, syntaxLine: syntaxLine }))));
    }
});
_DiffUnifiedLine.displayName = "_DiffUnifiedLine";
const DiffUnifiedLine = ({ index, diffFile, lineNumber, }) => {
    const unifiedLine = diffFile.getUnifiedLine(index);
    if (unifiedLine?.isHidden)
        return null;
    return React.createElement(_DiffUnifiedLine, { index: index, diffFile: diffFile, lineNumber: lineNumber });
};

const _DiffUnifiedWidgetLine = ({ index, diffFile, oldWidget, newWidget, lineNumber, setWidget, }) => {
    const unifiedItem = diffFile.getUnifiedLine(index);
    const onClose = () => setWidget({});
    const { useDiffContext } = useDiffViewContext();
    // 需要显示的时候才进行方法订阅，可以大幅度提高性能
    const renderWidgetLine = useDiffContext(useCallback((s) => s.renderWidgetLine, []));
    const width = useDomWidth({
        selector: ".unified-diff-table-wrapper",
        enable: typeof renderWidgetLine === "function",
    });
    if (!renderWidgetLine)
        return null;
    return (React.createElement("tr", { "data-line": `${lineNumber}-widget`, "data-state": "widget", className: "diff-line diff-line-widget" },
        React.createElement("td", { className: "diff-line-widget-content p-0", colSpan: 2 },
            React.createElement("div", { className: "diff-line-widget-wrapper sticky left-0", style: { width } },
                width > 0 &&
                    oldWidget &&
                    renderWidgetLine?.({ diffFile, side: SplitSide.old, lineNumber: unifiedItem.oldLineNumber, onClose }),
                width > 0 &&
                    newWidget &&
                    renderWidgetLine?.({ diffFile, side: SplitSide.new, lineNumber: unifiedItem.newLineNumber, onClose })))));
};
const DiffUnifiedWidgetLine = ({ index, diffFile, lineNumber, }) => {
    const { useWidget } = useDiffWidgetContext();
    const { widgetSide, widgetLineNumber, setWidget } = useWidget(React.useCallback((s) => ({ widgetLineNumber: s.widgetLineNumber, widgetSide: s.widgetSide, setWidget: s.setWidget }), []));
    const unifiedItem = diffFile.getUnifiedLine(index);
    const oldWidget = unifiedItem.oldLineNumber && widgetSide === SplitSide.old && widgetLineNumber === unifiedItem.oldLineNumber;
    const newWidget = unifiedItem.newLineNumber && widgetSide === SplitSide.new && widgetLineNumber === unifiedItem.newLineNumber;
    const currentIsShow = oldWidget || newWidget;
    if (!currentIsShow)
        return null;
    return (React.createElement(_DiffUnifiedWidgetLine, { index: index, diffFile: diffFile, lineNumber: lineNumber, oldWidget: oldWidget, newWidget: newWidget, setWidget: setWidget }));
};

const onMouseDown = (e) => {
    const ele = e.target;
    // need remove all the selection
    if (ele && ele instanceof HTMLElement && ele.nodeName === "BUTTON") {
        removeAllSelection();
        return;
    }
};
const DiffUnifiedView = memo(({ diffFile }) => {
    const { useDiffContext } = useDiffViewContext();
    // performance optimization
    const useWidget = useMemo(() => createStore(() => {
        const widgetSide = ref(undefined);
        const widgetLineNumber = ref(undefined);
        const setWidget = ({ side, lineNumber }) => {
            const { renderWidgetLine } = useDiffContext.getReadonlyState();
            if (typeof renderWidgetLine !== "function")
                return;
            widgetSide.value = side;
            widgetLineNumber.value = lineNumber;
        };
        return { widgetSide, widgetLineNumber, setWidget };
    }), [useDiffContext]);
    const contextValue = useMemo(() => ({ useWidget }), [useWidget]);
    useSyncExternalStore(diffFile.subscribe, diffFile.getUpdateCount);
    useEffect(() => {
        const { setWidget } = useWidget.getReadonlyState();
        setWidget({});
    }, [diffFile, useWidget]);
    const lines = getUnifiedContentLine(diffFile);
    return (React.createElement(DiffWidgetContext.Provider, { value: contextValue },
        React.createElement("div", { className: "unified-diff-view w-full" },
            React.createElement("div", { className: "unified-diff-table-wrapper overflow-auto w-full scrollbar-hide scrollbar-disable", style: { fontFamily: "Menlo, Consolas, monospace", fontSize: "var(--diff-font-size--)" } },
                React.createElement("table", { className: "unified-diff-table border-collapse w-full" },
                    React.createElement("colgroup", null,
                        React.createElement("col", { className: "unified-diff-table-num-col" }),
                        React.createElement("col", { className: "unified-diff-table-content-col" })),
                    React.createElement("thead", { className: "hidden" },
                        React.createElement("tr", null,
                            React.createElement("th", { scope: "col" }, "line number"),
                            React.createElement("th", { scope: "col" }, "line content"))),
                    React.createElement("tbody", { className: "diff-table-body  leading-[1.4]", onMouseDownCapture: onMouseDown },
                        lines.map((item) => (React.createElement(Fragment, { key: item.index },
                            React.createElement(DiffUnifiedHunkLine, { index: item.index, lineNumber: item.lineNumber, diffFile: diffFile }),
                            React.createElement(DiffUnifiedLine, { index: item.index, lineNumber: item.lineNumber, diffFile: diffFile }),
                            React.createElement(DiffUnifiedWidgetLine, { index: item.index, lineNumber: item.lineNumber, diffFile: diffFile }),
                            React.createElement(DiffUnifiedExtendLine, { index: item.index, lineNumber: item.lineNumber, diffFile: diffFile })))),
                        React.createElement(DiffUnifiedHunkLine, { index: diffFile.unifiedLineLength, lineNumber: diffFile.unifiedLineLength, diffFile: diffFile })))))));
});
DiffUnifiedView.displayName = "DiffUnifiedView";

/* eslint-disable @typescript-eslint/no-unnecessary-type-constraint */
/* eslint-disable @typescript-eslint/ban-ts-comment */
const diffFontSizeName = "--diff-font-size--";
var SplitSide;
(function (SplitSide) {
    SplitSide[SplitSide["old"] = 1] = "old";
    SplitSide[SplitSide["new"] = 2] = "new";
})(SplitSide || (SplitSide = {}));
const _InternalDiffView = (props) => {
    const { diffFile, className, style, diffViewMode, diffViewWrap, diffViewFontSize, diffViewHighlight, renderWidgetLine, renderExtendLine, extendData, diffViewAddWidget, onAddWidgetClick, } = props;
    const diffFileId = useMemo(() => diffFile.getId(), [diffFile]);
    // performance optimization
    const useDiffContext = useMemo(() => createStore(() => {
        const id = ref(diffFileId);
        const setId = (_id) => (id.value = _id);
        const mode = ref(props.diffViewMode);
        const setMode = (_mode) => (mode.value = _mode);
        const enableWrap = ref(props.diffViewWrap);
        const setEnableWrap = (_enableWrap) => (enableWrap.value = _enableWrap);
        const enableAddWidget = ref(props.diffViewAddWidget);
        const setEnableAddWidget = (_enableAddWidget) => (enableAddWidget.value = _enableAddWidget);
        const enableHighlight = ref(props.diffViewHighlight);
        const setEnableHighlight = (_enableHighlight) => (enableHighlight.value = _enableHighlight);
        const fontSize = ref(props.diffViewFontSize);
        const setFontSize = (_fontSize) => (fontSize.value = _fontSize);
        const extendData = ref({
            oldFile: { ...props.extendData?.oldFile },
            newFile: { ...props.extendData?.newFile },
        });
        const setExtendData = (_extendData) => {
            const existOldKeys = Object.keys(extendData.value.oldFile || {});
            const inComingOldKeys = Object.keys(_extendData.oldFile || {});
            for (const key of existOldKeys) {
                if (!inComingOldKeys.includes(key)) {
                    delete extendData.value.oldFile[key];
                }
            }
            for (const key of inComingOldKeys) {
                extendData.value.oldFile[key] = _extendData.oldFile[key];
            }
            const existNewKeys = Object.keys(extendData.value.newFile || {});
            const inComingNewKeys = Object.keys(_extendData.newFile || {});
            for (const key of existNewKeys) {
                if (!inComingNewKeys.includes(key)) {
                    delete extendData.value.newFile[key];
                }
            }
            for (const key of inComingNewKeys) {
                extendData.value.newFile[key] = _extendData.newFile[key];
            }
        };
        const renderWidgetLine = ref(props.renderWidgetLine);
        const setRenderWidgetLine = (_renderWidgetLine) => (renderWidgetLine.value = _renderWidgetLine);
        const renderExtendLine = ref(props.renderExtendLine);
        const setRenderExtendLine = (_renderExtendLine) => (renderExtendLine.value = _renderExtendLine);
        const onAddWidgetClick = ref(props.onAddWidgetClick);
        const setOnAddWidgetClick = (_onAddWidgetClick) => (onAddWidgetClick.value = _onAddWidgetClick);
        return {
            id,
            setId,
            mode,
            setMode,
            enableWrap,
            setEnableWrap,
            enableAddWidget,
            setEnableAddWidget,
            enableHighlight,
            setEnableHighlight,
            fontSize,
            setFontSize,
            extendData,
            setExtendData,
            renderWidgetLine,
            setRenderWidgetLine,
            renderExtendLine,
            setRenderExtendLine,
            onAddWidgetClick,
            setOnAddWidgetClick,
        };
    }), 
    // eslint-disable-next-line react-hooks/exhaustive-deps
    []);
    useEffect(() => {
        const { id, setId, mode, setMode, enableAddWidget, setEnableAddWidget, enableHighlight, setEnableHighlight, enableWrap, setEnableWrap, setExtendData, fontSize, setFontSize, onAddWidgetClick: _onAddWidgetClick, setOnAddWidgetClick, renderExtendLine: _renderExtendLine, setRenderExtendLine, renderWidgetLine: _renderWidgetLine, setRenderWidgetLine, } = useDiffContext.getReadonlyState();
        diffFileId && diffFileId !== id && setId(diffFileId);
        diffViewMode && diffViewMode !== mode && setMode(diffViewMode);
        diffViewAddWidget !== enableAddWidget && setEnableAddWidget(diffViewAddWidget);
        diffViewHighlight !== enableHighlight && setEnableHighlight(diffViewHighlight);
        diffViewWrap !== enableWrap && setEnableWrap(diffViewWrap);
        extendData && setExtendData(extendData);
        diffViewFontSize && diffViewFontSize !== fontSize && setFontSize(diffViewFontSize);
        onAddWidgetClick !== _onAddWidgetClick && setOnAddWidgetClick(onAddWidgetClick);
        renderExtendLine !== _renderExtendLine && setRenderExtendLine(renderExtendLine);
        renderWidgetLine !== _renderWidgetLine && setRenderWidgetLine(renderWidgetLine);
    }, [
        useDiffContext,
        diffViewFontSize,
        diffViewHighlight,
        diffViewMode,
        diffViewWrap,
        diffViewAddWidget,
        diffFileId,
        renderWidgetLine,
        renderExtendLine,
        extendData,
        onAddWidgetClick,
    ]);
    const value = useMemo(() => ({ useDiffContext }), [useDiffContext]);
    return (React.createElement(DiffViewContext.Provider, { value: value },
        React.createElement("div", { className: "diff-tailwindcss-wrapper", "data-component": "git-diff-view", "data-version": `${"0.0.6"}` },
            React.createElement("div", { className: "diff-style-root", style: {
                    // @ts-ignore
                    [diffFontSizeName]: diffViewFontSize + "px",
                } },
                React.createElement("div", { id: `diff-root${diffFileId}`, className: "diff-view-wrapper" + (className ? ` ${className}` : ""), style: style }, diffViewMode === DiffModeEnum.Split ? (React.createElement(DiffSplitView, { diffFile: diffFile })) : (React.createElement(DiffUnifiedView, { diffFile: diffFile })))))));
};
const InternalDiffView = memo(_InternalDiffView);
const DiffViewWithRef = (props, ref) => {
    const { registerHighlighter, autoDetectLang, data, diffFile: _diffFile, ...restProps } = props;
    const diffFile = useMemo(() => {
        if (_diffFile) {
            const diffFile = DiffFile.createInstance({});
            diffFile._mergeFullBundle(_diffFile._getFullBundle());
            return diffFile;
        }
        else if (data) {
            return new DiffFile(data?.oldFile?.fileName || "", data?.oldFile?.content || "", data?.newFile?.fileName || "", data?.newFile?.content || "", data?.hunks || [], data?.oldFile?.fileLang || "", data?.newFile?.fileLang || "");
        }
        return null;
    }, [data, _diffFile]);
    useEffect(() => {
        if (!diffFile)
            return;
        diffFile.initRaw();
        diffFile.buildSplitDiffLines();
        diffFile.buildUnifiedDiffLines();
    }, [diffFile]);
    useEffect(() => {
        if (!diffFile)
            return;
        if (props.diffViewHighlight) {
            diffFile.initSyntax({ autoDetectLang, registerHighlighter });
            diffFile.notifyAll();
        }
    }, [diffFile, props.diffViewHighlight, autoDetectLang, registerHighlighter]);
    useEffect(() => {
        if (_diffFile && diffFile) {
            _diffFile._addClonedInstance(diffFile);
            return () => {
                _diffFile._delClonedInstance(diffFile);
            };
        }
    }, [diffFile, _diffFile]);
    useUnmount(() => diffFile._destroy(), [diffFile]);
    useImperativeHandle(ref, () => ({ getDiffFileInstance: () => diffFile }), [diffFile]);
    if (!diffFile)
        return null;
    return (React.createElement(InternalDiffView, { key: diffFile.getId(), ...restProps, diffFile: diffFile, diffViewFontSize: restProps.diffViewFontSize || 14 }));
};
const DiffView = forwardRef(DiffViewWithRef);
DiffView.displayName = "DiffView";
const version = "0.0.6";

export { DiffModeEnum, DiffView, DiffViewContext, SplitSide, useDiffViewContext, version };
//# sourceMappingURL=index.mjs.map
